@@ -955,10 +955,6 @@ static inline int nv_io_remap_page_range(struct vm_area_struct *vma,
     return ret;
 }
 
-#ifndef NV_VMF_INSERT_PFN_PRESENT
-typedef int vm_fault_t;
-#endif
-
 static inline vm_fault_t nv_insert_pfn(struct vm_area_struct *vma,
     NvU64 virt_addr, NvU64 pfn, NvU32 extra_prot)
 {
