@@ -20,6 +20,7 @@
 #include <linux/random.h>
 #include <linux/file.h>
 #include <linux/list.h>
+#include <linux/rwsem.h>
 
 #include "nvstatus.h"
 
@@ -37,6 +38,15 @@
 
 #include "nv-time.h"
 
+#include <linux/sched.h>
+#if defined(NV_LINUX_SCHED_SIGNAL_H_PRESENT)
+#include <linux/sched/signal.h>
+#endif
+
+#if !defined(CONFIG_RETPOLINE)
+#include "nv-retpoline.h"
+#endif
+
 #if defined(NVCPU_X86_64) && defined(CONFIG_IA32_EMULATION) && \
   !defined(NV_FILE_OPERATIONS_HAS_COMPAT_IOCTL)
 #  define NV_NEEDS_COMPAT_IOCTL_REGISTRATION 1
@@ -78,10 +88,22 @@ static inline unsigned long NVKMS_USECS_TO_JIFFIES(NvU64 usec)
 static struct semaphore nvkms_lock;
 
 /*************************************************************************
- * NVKMS executes queued work items on a single kthread.
+ * User clients of NVKMS may need to be synchronized with suspend/resume
+ * operations.  This depends on the state of the system when the NVKMS
+ * suspend/resume callbacks are invoked.  NVKMS uses a single
+ * RW lock, nvkms_pm_lock, for this synchronization.
+ *************************************************************************/
+
+static struct rw_semaphore nvkms_pm_lock;
+
+/*************************************************************************
+ * NVKMS executes almost all of its queued work items on a single
+ * kthread.  The exception are deferred close() handlers, which typically
+ * block for long periods of time and stall their queue.
  *************************************************************************/
 
 static struct nv_kthread_q nvkms_kthread_q;
+static struct nv_kthread_q nvkms_deferred_close_kthread_q;
 
 /*************************************************************************
  * The nvkms_per_open structure tracks data that is specific to a
@@ -107,8 +129,53 @@ struct nvkms_per_open {
             } events;
         } kernel;
     } u;
+
+    nv_kthread_q_item_t deferred_close_q_item;
 };
 
+/*************************************************************************
+ * nvkms_pm_lock helper functions.  Since no down_read_interruptible()
+ * or equivalent interface is available, it needs to be approximated with
+ * down_read_trylock() to enable the kernel's freezer to round up user
+ * threads going into suspend.
+ *************************************************************************/
+
+static inline int nvkms_read_trylock_pm_lock(void)
+{
+    return !down_read_trylock(&nvkms_pm_lock);
+}
+
+static inline int nvkms_read_lock_pm_lock_interruptible(void)
+{
+    while (!down_read_trylock(&nvkms_pm_lock)) {
+        if (signal_pending(current)) {
+            return -EINTR;
+        }
+        cond_resched();
+    }
+
+    return 0;
+}
+
+static inline void nvkms_read_lock_pm_lock(void)
+{
+    down_read(&nvkms_pm_lock);
+}
+
+static inline void nvkms_read_unlock_pm_lock(void)
+{
+    up_read(&nvkms_pm_lock);
+}
+
+static inline void nvkms_write_lock_pm_lock(void)
+{
+    down_write(&nvkms_pm_lock);
+}
+
+static inline void nvkms_write_unlock_pm_lock(void)
+{
+    up_write(&nvkms_pm_lock);
+}
 
 /*************************************************************************
  * nvidia-modeset-os-interface.h functions.  It is assumed that these
@@ -322,6 +389,10 @@ nvkms_event_queue_changed(nvkms_per_open_handle_t *pOpenKernel,
 
 static void nvkms_suspend(NvU32 gpuId)
 {
+    if (gpuId == 0) {
+        nvkms_write_lock_pm_lock();
+    }
+
     down(&nvkms_lock);
     nvKmsSuspend(gpuId);
     up(&nvkms_lock);
@@ -332,6 +403,10 @@ static void nvkms_resume(NvU32 gpuId)
     down(&nvkms_lock);
     nvKmsResume(gpuId);
     up(&nvkms_lock);
+
+    if (gpuId == 0) {
+        nvkms_write_unlock_pm_lock();
+    }
 }
 
 
@@ -942,6 +1017,16 @@ void NVKMS_API_CALL nvkms_sema_up(nvkms_sema_handle_t *sema)
 
 #if defined(CONFIG_PROC_FS)
 
+#if defined(NV_DEFINE_SINGLE_PROCFS_FILE)
+#define NV_DEFINE_SINGLE_NVKMS_PROCFS_FILE(name) \
+    NV_DEFINE_SINGLE_PROCFS_FILE(name, \
+                                 nvkms_read_lock_pm_lock_interruptible, \
+                                 nvkms_read_unlock_pm_lock)
+#else
+#define NV_DEFINE_SINGLE_NVKMS_PROCFS_FILE(name) \
+    NV_DEFINE_PROCFS_SINGLE_FILE(name)
+#endif
+
 #define NVKMS_PROCFS_FOLDER "driver/nvidia-modeset"
 
 struct proc_dir_entry *nvkms_proc_dir;
@@ -985,7 +1070,7 @@ static int nv_procfs_read_nvkms_proc(struct seq_file *s, void *arg)
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(nvkms_proc);
+NV_DEFINE_SINGLE_NVKMS_PROCFS_FILE(nvkms_proc);
 
 static NvBool
 nvkms_add_proc_file(const nvkms_procfs_file_t *file)
@@ -996,11 +1081,8 @@ nvkms_add_proc_file(const nvkms_procfs_file_t *file)
         return NV_FALSE;
     }
 
-    new_proc_dir = NV_CREATE_PROC_ENTRY(file->name,
-                                        0,
-                                        nvkms_proc_dir,
-                                        &nv_procfs_nvkms_proc_fops,
-                                        file->func);
+    new_proc_dir = proc_create_data(file->name, 0, nvkms_proc_dir,
+                                    &nv_procfs_nvkms_proc_fops, file->func);
     return (new_proc_dir != NULL);
 }
 
@@ -1086,23 +1168,52 @@ EXPORT_SYMBOL(nvKmsKapiGetFunctionsTable);
 
 static int nvkms_open(struct inode *inode, struct file *filp)
 {
-    int status = 0;
+    int status;
+
+    status = nvkms_read_lock_pm_lock_interruptible();
+    if (status != 0) {
+        return status;
+    }
 
     filp->private_data =
         nvkms_open_common(NVKMS_CLIENT_USER_SPACE, NULL, &status);
 
+    nvkms_read_unlock_pm_lock();
+
     return status;
 }
 
+static void nvkms_close_deferred(void *data)
+{
+    struct nvkms_per_open *popen = data;
+
+    nvkms_read_lock_pm_lock();
+
+    nvkms_close_common(popen);
+
+    nvkms_read_unlock_pm_lock();
+}
+
 static int nvkms_close(struct inode *inode, struct file *filp)
 {
+    int status;
     struct nvkms_per_open *popen = filp->private_data;
 
     if (popen == NULL) {
         return -EINVAL;
     }
 
-    nvkms_close_common(popen);
+    status = nvkms_read_trylock_pm_lock();
+    if (status == 0) {
+        nvkms_close_common(popen);
+        nvkms_read_unlock_pm_lock();
+    } else {
+        nv_kthread_q_item_init(&popen->deferred_close_q_item,
+                               nvkms_close_deferred,
+                               popen);
+        nvkms_queue_work(&nvkms_deferred_close_kthread_q,
+                         &popen->deferred_close_q_item);
+    }
 
     return 0;
 }
@@ -1117,7 +1228,7 @@ static int nvkms_ioctl(struct inode *inode, struct file *filp,
 {
     size_t size;
     unsigned int nr;
-    int status = 0;
+    int status;
     struct NvKmsIoctlParams params;
     struct nvkms_per_open *popen = filp->private_data;
 
@@ -1139,10 +1250,19 @@ static int nvkms_ioctl(struct inode *inode, struct file *filp,
         return -EFAULT;
     }
 
-    return nvkms_ioctl_common(popen,
-                              params.cmd,
-                              params.address,
-                              params.size);
+    status = nvkms_read_lock_pm_lock_interruptible();
+    if (status != 0) {
+        return status;
+    }
+
+    status = nvkms_ioctl_common(popen,
+                                params.cmd,
+                                params.address,
+                                params.size);
+
+    nvkms_read_unlock_pm_lock();
+
+    return status;
 }
 
 static void nvkms_register_compatible_ioctl(void)
@@ -1208,18 +1328,29 @@ static int __init nvkms_init(void)
     }
 
     sema_init(&nvkms_lock, 1);
+    init_rwsem(&nvkms_pm_lock);
 
-    ret = nv_kthread_q_init(&nvkms_kthread_q, "nvidia-modeset");
+    ret = nv_kthread_q_init(&nvkms_kthread_q,
+                            "nvidia-modeset/kthread_q");
     if (ret != 0) {
         return ret;
     }
 
+    ret = nv_kthread_q_init(&nvkms_deferred_close_kthread_q,
+                            "nvidia-modeset/deferred_close_kthread_q");
+    if (ret != 0) {
+        nv_kthread_q_stop(&nvkms_kthread_q);
+        return ret;
+    }
+
     INIT_LIST_HEAD(&nvkms_timers.list);
     spin_lock_init(&nvkms_timers.lock);
 
     ret = nvidia_register_module(&nvidia_modeset_module);
 
     if (ret != 0) {
+        nv_kthread_q_stop(&nvkms_deferred_close_kthread_q);
+        nv_kthread_q_stop(&nvkms_kthread_q);
         return ret;
     }
 
@@ -1285,6 +1416,7 @@ restart:
 
     spin_unlock_irqrestore(&nvkms_timers.lock, flags);
 
+    nv_kthread_q_stop(&nvkms_deferred_close_kthread_q);
     nv_kthread_q_stop(&nvkms_kthread_q);
 
     nvkms_unregister_compatible_ioctl();
