@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2018 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -459,10 +459,10 @@ typedef struct
     } stats;
 
     // Ignoring access counters means that notifications are left in the HW
-    // buffer without being serviced. This is used by tests to force a
-    // notification buffer overflow, and it's used to temporarily avoid
-    // notification processing during GPU unregister.
-    bool ignore_notifications;
+    // buffer without being serviced.  Requests to ignore access counters
+    // are counted since the suspend path inhibits access counter interrupts,
+    // and the resume path needs to know whether to reenable them.
+    NvU32 notifications_ignored_count;
 
     // Context structure used to service a GPU access counter batch
     uvm_access_counter_service_batch_context_t batch_service_context;
@@ -491,6 +491,9 @@ typedef enum
     UVM_GPU_LINK_PCIE,
     UVM_GPU_LINK_NVLINK_1,
     UVM_GPU_LINK_NVLINK_2,
+
+
+
     UVM_GPU_LINK_MAX
 } uvm_gpu_link_type_t;
 
@@ -698,6 +701,10 @@ struct uvm_gpu_struct
     // This value is only defined for GPUs that support non-replayable faults
     bool has_clear_faulted_channel_method;
 
+
+
+
+
     // Parameters used by the TLB batching API
     struct
     {
@@ -740,6 +747,14 @@ struct uvm_gpu_struct
     NvU64 uvm_mem_va_base;
     NvU64 uvm_mem_va_size;
 
+    // Bitmap of allocation sizes for user memory supported by a GPU. PAGE_SIZE
+    // is guaranteed to be both present and the smallest size.
+    uvm_chunk_sizes_mask_t mmu_user_chunk_sizes;
+
+    // Bitmap of allocation sizes that could be requested by the page tree for
+    // a GPU
+    uvm_chunk_sizes_mask_t mmu_kernel_chunk_sizes;
+
     // RM address space handle used in many of the UVM/RM APIs
     // Represents both an RM device and a GPU VA in RM.
     uvmGpuAddressSpaceHandle rm_address_space;
@@ -994,11 +1009,8 @@ uvm_gpu_t *uvm_gpu_get_by_uuid_locked(const NvProcessorUuid *gpu_uuid);
 
 // Retain a gpu by uuid
 // Returns the retained uvm_gpu_t in gpu_out on success
-// LOCKING: requires the global lock to be held
-NV_STATUS uvm_gpu_retain_by_uuid_locked(const NvProcessorUuid *gpu_uuid, uvm_gpu_t **gpu_out);
-
-// Like uvm_gpu_retain_by_uuid_locked, but takes and releases the global lock
-// for the caller.
+//
+// LOCKING: Takes and releases the global lock for the caller.
 NV_STATUS uvm_gpu_retain_by_uuid(const NvProcessorUuid *gpu_uuid, uvm_gpu_t **gpu_out);
 
 // Retain a gpu which is known to already be retained. Does NOT require the
@@ -1021,6 +1033,9 @@ void uvm_gpu_release_mask_locked(const uvm_processor_mask_t *mask);
 
 // Like uvm_gpu_release_mask_locked, but takes and releases the global lock for
 // the caller.
+//
+// LOCKING: this function takes and releases the global lock if the input mask
+//          is not empty
 void uvm_gpu_release_mask(const uvm_processor_mask_t *mask);
 
 static NvU64 uvm_gpu_retained_count(uvm_gpu_t *gpu)
@@ -1063,7 +1078,7 @@ static uvm_gpu_peer_t *uvm_gpu_peer_caps(const uvm_gpu_t *gpu_1, const uvm_gpu_t
 static bool uvm_gpus_are_nvswitch_connected(uvm_gpu_t *gpu1, uvm_gpu_t *gpu2)
 {
     if (gpu1->nvswitch_info.is_nvswitch_connected && gpu2->nvswitch_info.is_nvswitch_connected) {
-        UVM_ASSERT(uvm_gpu_peer_caps(gpu1, gpu2)->link_type == UVM_GPU_LINK_NVLINK_2);
+        UVM_ASSERT(uvm_gpu_peer_caps(gpu1, gpu2)->link_type >= UVM_GPU_LINK_NVLINK_2);
         return true;
     }
 
