@@ -195,6 +195,7 @@ NV_STATUS uvm8_isr_top_half(const NvProcessorUuid *gpu_uuid)
 {
     uvm_gpu_t *gpu;
     unsigned num_handlers_scheduled = 0;
+    NV_STATUS status;
 
     if (!in_interrupt()) {
         // Early-out if not in interrupt context. This happens with
@@ -235,18 +236,25 @@ NV_STATUS uvm8_isr_top_half(const NvProcessorUuid *gpu_uuid)
     // Now that we got a GPU object, lock it so that it can't be removed without us noticing.
     uvm_spin_lock_irqsave(&gpu->isr.interrupts_lock);
 
-    num_handlers_scheduled += schedule_replayable_faults_handler(gpu);
-    num_handlers_scheduled += schedule_non_replayable_faults_handler(gpu);
-    num_handlers_scheduled += schedule_access_counters_handler(gpu);
+    if (gpu->isr.is_suspended) {
+        status = NV_ERR_NO_INTR_PENDING;
+    }
+    else {
+        num_handlers_scheduled += schedule_replayable_faults_handler(gpu);
+        num_handlers_scheduled += schedule_non_replayable_faults_handler(gpu);
+        num_handlers_scheduled += schedule_access_counters_handler(gpu);
+
+        if (num_handlers_scheduled == 0)
+            status = NV_WARN_MORE_PROCESSING_REQUIRED;
+        else
+            status = NV_OK;
+    }
 
     uvm_spin_unlock_irqrestore(&gpu->isr.interrupts_lock);
 
     uvm_gpu_kref_put(gpu);
 
-    if (num_handlers_scheduled == 0)
-        return NV_WARN_MORE_PROCESSING_REQUIRED;
-    else
-        return NV_OK;
+    return status;
 }
 
 NV_STATUS uvm_gpu_init_isr(uvm_gpu_t *gpu)
