@@ -33,6 +33,33 @@
 // Any locks described here should have their locking order added to
 // uvm_lock_order_t below.
 //
+// - Global power management lock (g_uvm_global.pm.lock)
+//      Order: UVM_LOCK_ORDER_GLOBAL_PM
+//      Reader/write lock (rw_semaphore)
+//
+//      Synchronizes user threads with system power management.
+//
+//      Taken in read mode by most user-facing UVM driver entry points.  Taken
+//      in write mode by uvm_suspend(), only, and held for the duration of
+//      sleep cycles.
+//
+//      This lock is special: while it's taken by user-facing entry points,
+//      and may be taken before or after mmap_sem, this apparent violation of
+//      lock ordering is permissible because pm_lock may only be taken via
+//      trylock in read mode by paths which already hold any lower-level
+//      locks, as well as by paths subject to the kernel's freezer.  Paths
+//      taking it must be prepared to back off in case of acquisition failures.
+//
+//      This, in turn, is acceptable because the lock is taken in write mode
+//      infrequently, and only as part of to power management.  Starvation is
+//      not a concern.
+//
+//      The mmap_sem deadlock potential aside, the trylock approch is also
+//      motivated by the need to prevent user threads making UVM system calls
+//      from blocking when UVM is suspended: when the kernel suspends the
+//      system, the freezer employed to stop user tasks requires these tasks
+//      to be interruptible.
+//
 // - ATS IBM GPU VA space register/unregister lock
 //      Order: UVM_LOCK_ORDER_ATS_IBM_REG_UNREG
 //      Exclusive lock (mutex)
@@ -139,6 +166,10 @@
 //      UVM driver calls into RM. In other words, mmap_sem and the RM GPUs lock
 //      are mutually exclusive.
 //
+// - Global VA spaces list lock
+//      Order: UVM_LOCK_ORDER_VA_SPACES_LIST
+//      Mutex which protects g_uvm_global.va_spaces state.
+//
 // - VA space writer serialization lock (va_space->serialize_writers_lock)
 //      Order: UVM_LOCK_ORDER_VA_SPACE_SERIALIZE_WRITERS
 //      Exclusive lock (mutex) per uvm_va_space (UVM struct file)
@@ -350,10 +381,6 @@
 //      events come from perf events, both VA_SPACE_EVENTS and VA_SPACE_TOOLS
 //      must be taken to register/report some tools events.
 //
-// - Global VA spaces list lock
-//      Order: UVM_LOCK_ORDER_VA_SPACES_LIST
-//      Mutex which protects g_uvm_global.va_spaces state.
-//
 // - Leaf locks
 //      Order: UVM_LOCK_ORDER_LEAF
 //
@@ -365,11 +392,13 @@
 typedef enum
 {
     UVM_LOCK_ORDER_INVALID = 0,
+    UVM_LOCK_ORDER_GLOBAL_PM,
     UVM_LOCK_ORDER_ATS_IBM_REG_UNREG,
     UVM_LOCK_ORDER_ATS_IBM_MM,
     UVM_LOCK_ORDER_GLOBAL,
     UVM_LOCK_ORDER_ISR,
     UVM_LOCK_ORDER_MMAP_SEM,
+    UVM_LOCK_ORDER_VA_SPACES_LIST,
     UVM_LOCK_ORDER_VA_SPACE_SERIALIZE_WRITERS,
     UVM_LOCK_ORDER_VA_SPACE_READ_ACQUIRE_WRITE_RELEASE_LOCK,
     UVM_LOCK_ORDER_VA_SPACE,
@@ -388,7 +417,6 @@ typedef enum
     UVM_LOCK_ORDER_VA_SPACE_EVENTS,
     UVM_LOCK_ORDER_VA_SPACE_TOOLS,
     UVM_LOCK_ORDER_SEMA_POOL_TRACKER,
-    UVM_LOCK_ORDER_VA_SPACES_LIST,
     UVM_LOCK_ORDER_LEAF,
     UVM_LOCK_ORDER_COUNT,
 } uvm_lock_order_t;
@@ -397,34 +425,39 @@ const char *uvm_lock_order_to_string(uvm_lock_order_t lock_order);
 
 typedef enum
 {
-    UVM_LOCK_MODE_EXCLUSIVE,
-    UVM_LOCK_MODE_SHARED,
+    UVM_LOCK_FLAGS_INVALID          = 0,
+    UVM_LOCK_FLAGS_MODE_EXCLUSIVE   = (1 << 0),
+    UVM_LOCK_FLAGS_MODE_SHARED      = (1 << 1),
+    UVM_LOCK_FLAGS_MODE_ANY         = (UVM_LOCK_FLAGS_MODE_EXCLUSIVE | UVM_LOCK_FLAGS_MODE_SHARED),
+    UVM_LOCK_FLAGS_MODE_MASK        = (UVM_LOCK_FLAGS_MODE_EXCLUSIVE | UVM_LOCK_FLAGS_MODE_SHARED),
+    UVM_LOCK_FLAGS_OUT_OF_ORDER     = (1 << 2),
+    UVM_LOCK_FLAGS_TRYLOCK          = (1 << 3),
+    UVM_LOCK_FLAGS_MASK             = (1 << 4) - 1
+} uvm_lock_flags_t;
 
-    // Special value so uvm_check_locked can check for either mode
-    UVM_LOCK_MODE_ANY,
-} uvm_lock_mode_t;
-
-// Record locking a lock of given lock_order in exclusive or shared mode
+// Record locking a lock of given lock_order in exclusive or shared mode,
+// distinguishing between trylock and normal acquisition attempts.
 // Returns true if the recorded lock follows all the locking rules and false otherwise.
-bool __uvm_record_lock(void *lock, uvm_lock_order_t lock_order, uvm_lock_mode_t mode);
+bool __uvm_record_lock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t flags);
 
 // Record unlocking a lock of given lock_order in exclusive or shared mode and
 // possibly out of order.
 // Returns true if the unlock follows all the locking rules and false otherwise.
-bool __uvm_record_unlock(void *lock, uvm_lock_order_t lock_order, uvm_lock_mode_t mode, bool out_of_order);
+bool __uvm_record_unlock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t flags);
 
 bool __uvm_record_downgrade(void *lock, uvm_lock_order_t lock_order);
 
 // Check whether a lock of given lock_order is held in exclusive, shared, or
 // either mode by the current thread.
-bool __uvm_check_locked(void *lock, uvm_lock_order_t lock_order, uvm_lock_mode_t mode);
+bool __uvm_check_locked(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t flags);
 
 // Check that no locks are held with the given lock order
 bool __uvm_check_unlocked_order(uvm_lock_order_t lock_order);
 
 // Check that a lock of the given order can be locked, i.e. that no locks are
-// held with the given or deeper lock order.
-bool __uvm_check_lockable_order(uvm_lock_order_t lock_order);
+// held with the given or deeper lock order.  Allow for out-of-order locking
+// when checking for a trylock.
+bool __uvm_check_lockable_order(uvm_lock_order_t lock_order, uvm_lock_flags_t flags);
 
 // Check that all locks have been released in a uvm thread context
 bool __uvm_check_all_unlocked(uvm_thread_context_t *uvm_context);
@@ -438,56 +471,64 @@ bool __uvm_locking_initialized(void);
 #if UVM_IS_DEBUG()
   // These macros are intended to be expanded on the call site directly and will print
   // the precise location of the violation while the __uvm_record* functions will error print the details.
-  #define uvm_record_lock_raw(lock, lock_order, mode) \
-      UVM_ASSERT_MSG(__uvm_record_lock((lock), (lock_order), (mode)), "Locking violation\n")
-  #define uvm_record_unlock_raw(lock, lock_order, mode, out_of_order) \
-      UVM_ASSERT_MSG(__uvm_record_unlock((lock), (lock_order), (mode), (out_of_order)), "Locking violation\n")
+  #define uvm_record_lock_raw(lock, lock_order, flags) \
+      UVM_ASSERT_MSG(__uvm_record_lock((lock), (lock_order), (flags)), "Locking violation\n")
+  #define uvm_record_unlock_raw(lock, lock_order, flags) \
+      UVM_ASSERT_MSG(__uvm_record_unlock((lock), (lock_order), (flags)), "Locking violation\n")
   #define uvm_record_downgrade_raw(lock, lock_order) \
       UVM_ASSERT_MSG(__uvm_record_downgrade((lock), (lock_order)), "Locking violation\n")
 
   // Record UVM lock (a lock that has a lock_order member) operation and assert that it's correct
-  #define uvm_record_lock(lock, mode) uvm_record_lock_raw((lock), (lock)->lock_order, (mode))
-  #define uvm_record_unlock(lock, mode) uvm_record_unlock_raw((lock), (lock)->lock_order, (mode), false)
-  #define uvm_record_unlock_out_of_order(lock, mode) uvm_record_unlock_raw((lock), (lock)->lock_order, (mode), true)
+  #define uvm_record_lock(lock, flags) \
+      uvm_record_lock_raw((lock), (lock)->lock_order, (flags))
+  #define uvm_record_unlock(lock, flags) uvm_record_unlock_raw((lock), (lock)->lock_order, (flags))
+  #define uvm_record_unlock_out_of_order(lock, flags) \
+            uvm_record_unlock_raw((lock), (lock)->lock_order, (flags) | UVM_LOCK_FLAGS_OUT_OF_ORDER)
   #define uvm_record_downgrade(lock) uvm_record_downgrade_raw((lock), (lock)->lock_order)
 
   // Check whether a UVM lock (a lock that has a lock_order member) is held in
   // the given mode.
-  #define uvm_check_locked(lock, mode) __uvm_check_locked((lock), (lock)->lock_order, (mode))
+  #define uvm_check_locked(lock, flags) __uvm_check_locked((lock), (lock)->lock_order, (flags))
 
   // Helpers for recording and asserting mmap_sem state
   #define uvm_record_lock_mmap_sem_read(mmap_sem) \
-          uvm_record_lock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_MODE_SHARED)
+          uvm_record_lock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_FLAGS_MODE_SHARED)
 
   #define uvm_record_unlock_mmap_sem_read(mmap_sem) \
-          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_MODE_SHARED, false)
+          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_FLAGS_MODE_SHARED)
 
   #define uvm_record_unlock_mmap_sem_read_out_of_order(mmap_sem) \
-          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_MODE_SHARED, true)
+          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, \
+                                UVM_LOCK_FLAGS_MODE_SHARED | UVM_LOCK_FLAGS_OUT_OF_ORDER)
 
   #define uvm_record_lock_mmap_sem_write(mmap_sem) \
-          uvm_record_lock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_MODE_EXCLUSIVE)
+          uvm_record_lock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
 
   #define uvm_record_unlock_mmap_sem_write(mmap_sem) \
-          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_MODE_EXCLUSIVE, false)
+          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
 
   #define uvm_record_unlock_mmap_sem_write_out_of_order(mmap_sem) \
-          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, UVM_LOCK_MODE_EXCLUSIVE, true)
+          uvm_record_unlock_raw((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, \
+                                UVM_LOCK_FLAGS_MODE_EXCLUSIVE | UVM_LOCK_FLAGS_OUT_OF_ORDER)
 
-  #define uvm_check_locked_mmap_sem(mmap_sem, mode) \
-           __uvm_check_locked((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, (mode))
+  #define uvm_check_locked_mmap_sem(mmap_sem, flags) \
+           __uvm_check_locked((mmap_sem), UVM_LOCK_ORDER_MMAP_SEM, (flags))
 
   // Helpers for recording RM API lock usage around UVM-RM interfaces
   #define uvm_record_lock_rm_api() \
-          uvm_record_lock_raw((void*)UVM_LOCK_ORDER_RM_API, UVM_LOCK_ORDER_RM_API, UVM_LOCK_MODE_EXCLUSIVE)
+          uvm_record_lock_raw((void*)UVM_LOCK_ORDER_RM_API, UVM_LOCK_ORDER_RM_API, \
+                              UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
   #define uvm_record_unlock_rm_api() \
-          uvm_record_unlock_raw((void*)UVM_LOCK_ORDER_RM_API, UVM_LOCK_ORDER_RM_API, UVM_LOCK_MODE_EXCLUSIVE, false)
+          uvm_record_unlock_raw((void*)UVM_LOCK_ORDER_RM_API, UVM_LOCK_ORDER_RM_API, \
+                                UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
 
   // Helpers for recording RM GPUS lock usage around UVM-RM interfaces
   #define uvm_record_lock_rm_gpus() \
-          uvm_record_lock_raw((void*)UVM_LOCK_ORDER_RM_GPUS, UVM_LOCK_ORDER_RM_GPUS, UVM_LOCK_MODE_EXCLUSIVE)
+          uvm_record_lock_raw((void*)UVM_LOCK_ORDER_RM_GPUS, UVM_LOCK_ORDER_RM_GPUS, \
+                              UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
   #define uvm_record_unlock_rm_gpus() \
-          uvm_record_unlock_raw((void*)UVM_LOCK_ORDER_RM_GPUS, UVM_LOCK_ORDER_RM_GPUS, UVM_LOCK_MODE_EXCLUSIVE, false)
+          uvm_record_unlock_raw((void*)UVM_LOCK_ORDER_RM_GPUS, UVM_LOCK_ORDER_RM_GPUS, \
+                                UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
 
   // Helpers for recording both RM locks usage around UVM-RM interfaces
   #define uvm_record_lock_rm_all() ({ uvm_record_lock_rm_api(); uvm_record_lock_rm_gpus(); })
@@ -499,7 +540,7 @@ bool __uvm_locking_initialized(void);
   #define uvm_record_unlock_out_of_order                UVM_IGNORE_EXPR2
   #define uvm_record_downgrade                          UVM_IGNORE_EXPR
 
-  static bool uvm_check_locked(void *lock, uvm_lock_mode_t mode)
+  static bool uvm_check_locked(void *lock, uvm_lock_flags_t flags)
   {
       return false;
   }
@@ -525,18 +566,21 @@ bool __uvm_locking_initialized(void);
 
 #define uvm_locking_assert_initialized() UVM_ASSERT(__uvm_locking_initialized())
 #define uvm_thread_assert_all_unlocked() UVM_ASSERT(__uvm_thread_check_all_unlocked())
-#define uvm_assert_lockable_order(order) UVM_ASSERT(__uvm_check_lockable_order(order))
+#define uvm_assert_lockable_order(order) UVM_ASSERT(__uvm_check_lockable_order(order, UVM_LOCK_FLAGS_MODE_ANY))
 #define uvm_assert_unlocked_order(order) UVM_ASSERT(__uvm_check_unlocked_order(order))
 
 // Helpers for locking mmap_sem and recording its usage
-#define uvm_assert_mmap_sem_locked_mode(mmap_sem, mode) ({                          \
-      typeof(mmap_sem) _sem = (mmap_sem);                                           \
-      UVM_ASSERT(rwsem_is_locked(_sem) && uvm_check_locked_mmap_sem(_sem, (mode))); \
+#define uvm_assert_mmap_sem_locked_mode(mmap_sem, flags) ({                          \
+      typeof(mmap_sem) _sem = (mmap_sem);                                            \
+      UVM_ASSERT(rwsem_is_locked(_sem) && uvm_check_locked_mmap_sem(_sem, (flags))); \
   })
 
-#define uvm_assert_mmap_sem_locked(mmap_sem)        uvm_assert_mmap_sem_locked_mode((mmap_sem), UVM_LOCK_MODE_ANY)
-#define uvm_assert_mmap_sem_locked_read(mmap_sem)   uvm_assert_mmap_sem_locked_mode((mmap_sem), UVM_LOCK_MODE_SHARED)
-#define uvm_assert_mmap_sem_locked_write(mmap_sem)  uvm_assert_mmap_sem_locked_mode((mmap_sem), UVM_LOCK_MODE_EXCLUSIVE)
+#define uvm_assert_mmap_sem_locked(mmap_sem) \
+        uvm_assert_mmap_sem_locked_mode((mmap_sem), UVM_LOCK_FLAGS_MODE_ANY)
+#define uvm_assert_mmap_sem_locked_read(mmap_sem) \
+        uvm_assert_mmap_sem_locked_mode((mmap_sem), UVM_LOCK_FLAGS_MODE_SHARED)
+#define uvm_assert_mmap_sem_locked_write(mmap_sem) \
+        uvm_assert_mmap_sem_locked_mode((mmap_sem), UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
 
 #define uvm_down_read_mmap_sem(mmap_sem) ({             \
         typeof(mmap_sem) _sem = (mmap_sem);             \
@@ -592,14 +636,17 @@ typedef struct
 #endif
 } uvm_rw_semaphore_t;
 
-#define uvm_assert_rwsem_locked_mode(uvm_sem, mode) ({                              \
+#define uvm_assert_rwsem_locked_mode(uvm_sem, flags) ({                             \
         typeof(uvm_sem) _sem = (uvm_sem);                                           \
-        UVM_ASSERT(rwsem_is_locked(&_sem->sem) && uvm_check_locked(_sem, (mode)));  \
+        UVM_ASSERT(rwsem_is_locked(&_sem->sem) && uvm_check_locked(_sem, (flags))); \
     })
 
-#define uvm_assert_rwsem_locked(uvm_sem)        uvm_assert_rwsem_locked_mode(uvm_sem, UVM_LOCK_MODE_ANY)
-#define uvm_assert_rwsem_locked_read(uvm_sem)   uvm_assert_rwsem_locked_mode(uvm_sem, UVM_LOCK_MODE_SHARED)
-#define uvm_assert_rwsem_locked_write(uvm_sem)  uvm_assert_rwsem_locked_mode(uvm_sem, UVM_LOCK_MODE_EXCLUSIVE)
+#define uvm_assert_rwsem_locked(uvm_sem) \
+        uvm_assert_rwsem_locked_mode(uvm_sem, UVM_LOCK_FLAGS_MODE_ANY)
+#define uvm_assert_rwsem_locked_read(uvm_sem) \
+        uvm_assert_rwsem_locked_mode(uvm_sem, UVM_LOCK_FLAGS_MODE_SHARED)
+#define uvm_assert_rwsem_locked_write(uvm_sem) \
+        uvm_assert_rwsem_locked_mode(uvm_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
 
 #define uvm_assert_rwsem_unlocked(uvm_sem) UVM_ASSERT(!rwsem_is_locked(&(uvm_sem)->sem))
 
@@ -619,10 +666,10 @@ static void __uvm_down_read(uvm_rw_semaphore_t *uvm_sem)
     uvm_assert_rwsem_locked_read(uvm_sem);
 }
 
-#define uvm_down_read(sem) ({                           \
-        typeof(sem) _sem = (sem);                       \
-        uvm_record_lock(_sem, UVM_LOCK_MODE_SHARED);    \
-        __uvm_down_read(_sem);                          \
+#define uvm_down_read(sem) ({                              \
+        typeof(sem) _sem = (sem);                          \
+        uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
+        __uvm_down_read(_sem);                             \
     })
 
 static void __uvm_up_read(uvm_rw_semaphore_t *uvm_sem)
@@ -630,10 +677,10 @@ static void __uvm_up_read(uvm_rw_semaphore_t *uvm_sem)
     uvm_assert_rwsem_locked_read(uvm_sem);
     up_read(&uvm_sem->sem);
 }
-#define uvm_up_read(sem) ({                             \
-        typeof(sem) _sem = (sem);                       \
-        __uvm_up_read(_sem);                            \
-        uvm_record_unlock(_sem, UVM_LOCK_MODE_SHARED);  \
+#define uvm_up_read(sem) ({                                  \
+        typeof(sem) _sem = (sem);                            \
+        __uvm_up_read(_sem);                                 \
+        uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
     })
 
 static void __uvm_down_write(uvm_rw_semaphore_t *uvm_sem)
@@ -641,13 +688,42 @@ static void __uvm_down_write(uvm_rw_semaphore_t *uvm_sem)
     down_write(&uvm_sem->sem);
     uvm_assert_rwsem_locked_write(uvm_sem);
 }
-#define uvm_down_write(sem) ({                          \
-        typeof (sem) _sem = (sem);                      \
-        uvm_record_lock(_sem, UVM_LOCK_MODE_EXCLUSIVE); \
-        __uvm_down_write(_sem);                         \
+#define uvm_down_write(sem) ({                                \
+        typeof (sem) _sem = (sem);                            \
+        uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
+        __uvm_down_write(_sem);                               \
     })
 
-// trylock for writing: returns 1 if successful, 0 if not:
+// trylock for reading: returns 1 if successful, 0 if not.  Out-of-order lock
+// acquisition via this function is legal, i.e. the lock order checker will
+// allow it.  However, if an out-of-order lock acquisition attempt fails, it is
+// the caller's responsibility to back off at least to the point where the
+// next held lower-order lock is released.
+static int __uvm_down_read_trylock(uvm_rw_semaphore_t *uvm_sem)
+{
+    int ret = down_read_trylock(&uvm_sem->sem);
+    if (ret == 0)
+        return ret;
+
+    uvm_assert_rwsem_locked_read(uvm_sem);
+
+    return ret;
+}
+#define uvm_down_read_trylock(sem) ({                                               \
+        typeof(sem) _sem = (sem);                                                   \
+        int locked;                                                                 \
+        uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_SHARED | UVM_LOCK_FLAGS_TRYLOCK); \
+        locked = __uvm_down_read_trylock(_sem);                                     \
+        if (locked == 0)                                                            \
+            uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_SHARED);                    \
+        locked;                                                                     \
+    })
+
+// trylock for writing: returns 1 if successful, 0 if not.  Out-of-order lock
+// acquisition via this function is legal, i.e. the lock order checker will
+// allow it.  However, if an out-of-order lock acquisition attempt fails, it is
+// the caller's responsibility to back off at least to the point where the
+// next held lower-order lock is released.
 static int __uvm_down_write_trylock(uvm_rw_semaphore_t *uvm_sem)
 {
     int ret = down_write_trylock(&uvm_sem->sem);
@@ -658,14 +734,14 @@ static int __uvm_down_write_trylock(uvm_rw_semaphore_t *uvm_sem)
 
     return ret;
 }
-#define uvm_down_write_trylock(sem) ({                          \
-        typeof(sem) _sem = (sem);                               \
-        int locked;                                             \
-        uvm_record_lock(_sem, UVM_LOCK_MODE_EXCLUSIVE);         \
-        locked = __uvm_down_write_trylock(_sem);                \
-        if (locked == 0)                                        \
-            uvm_record_unlock(_sem, UVM_LOCK_MODE_EXCLUSIVE);   \
-        locked;                                                 \
+#define uvm_down_write_trylock(sem) ({                                                 \
+        typeof(sem) _sem = (sem);                                                      \
+        int locked;                                                                    \
+        uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE | UVM_LOCK_FLAGS_TRYLOCK); \
+        locked = __uvm_down_write_trylock(_sem);                                       \
+        if (locked == 0)                                                               \
+            uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE);                    \
+        locked;                                                                        \
     })
 
 static void __uvm_up_write(uvm_rw_semaphore_t *uvm_sem)
@@ -673,10 +749,10 @@ static void __uvm_up_write(uvm_rw_semaphore_t *uvm_sem)
     uvm_assert_rwsem_locked_write(uvm_sem);
     up_write(&uvm_sem->sem);
 }
-#define uvm_up_write(sem) ({                                \
-        typeof(sem) _sem = (sem);                           \
-        __uvm_up_write(_sem);                               \
-        uvm_record_unlock(_sem, UVM_LOCK_MODE_EXCLUSIVE);   \
+#define uvm_up_write(sem) ({                                    \
+        typeof(sem) _sem = (sem);                               \
+        __uvm_up_write(_sem);                                   \
+        uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
 static void __uvm_downgrade_write(uvm_rw_semaphore_t *uvm_sem)
@@ -698,9 +774,9 @@ typedef struct
 #endif
 } uvm_mutex_t;
 
-#define uvm_assert_mutex_locked(uvm_mutex) ({                                                           \
-        typeof(uvm_mutex) _mutex = (uvm_mutex);                                                         \
-        UVM_ASSERT(mutex_is_locked(&_mutex->m) && uvm_check_locked(_mutex, UVM_LOCK_MODE_EXCLUSIVE));   \
+#define uvm_assert_mutex_locked(uvm_mutex) ({                                                               \
+        typeof(uvm_mutex) _mutex = (uvm_mutex);                                                             \
+        UVM_ASSERT(mutex_is_locked(&_mutex->m) && uvm_check_locked(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
     })
 
 #define uvm_assert_mutex_unlocked(uvm_mutex) UVM_ASSERT(!mutex_is_locked(&(uvm_mutex)->m))
@@ -720,10 +796,10 @@ static void __uvm_mutex_lock(uvm_mutex_t *mutex)
     mutex_lock(&mutex->m);
     uvm_assert_mutex_locked(mutex);
 }
-#define uvm_mutex_lock(mutex) ({                            \
-        typeof(mutex) _mutex = (mutex);                     \
-        uvm_record_lock(_mutex, UVM_LOCK_MODE_EXCLUSIVE);   \
-        __uvm_mutex_lock(_mutex);                           \
+#define uvm_mutex_lock(mutex) ({                                \
+        typeof(mutex) _mutex = (mutex);                         \
+        uvm_record_lock(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
+        __uvm_mutex_lock(_mutex);                               \
     })
 
 // Lock w/o any tracking. This should be extremely rare and *_no_tracking
@@ -735,15 +811,15 @@ static void __uvm_mutex_unlock(uvm_mutex_t *mutex)
     uvm_assert_mutex_locked(mutex);
     mutex_unlock(&mutex->m);
 }
-#define uvm_mutex_unlock(mutex) ({                          \
-        typeof(mutex) _mutex = (mutex);                     \
-        __uvm_mutex_unlock(_mutex);                         \
-        uvm_record_unlock(_mutex, UVM_LOCK_MODE_EXCLUSIVE); \
+#define uvm_mutex_unlock(mutex) ({                                \
+        typeof(mutex) _mutex = (mutex);                           \
+        __uvm_mutex_unlock(_mutex);                               \
+        uvm_record_unlock(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
-#define uvm_mutex_unlock_out_of_order(mutex) ({                          \
-        typeof(mutex) _mutex = (mutex);                                  \
-        __uvm_mutex_unlock(_mutex);                                      \
-        uvm_record_unlock_out_of_order(_mutex, UVM_LOCK_MODE_EXCLUSIVE); \
+#define uvm_mutex_unlock_out_of_order(mutex) ({                                \
+        typeof(mutex) _mutex = (mutex);                                        \
+        __uvm_mutex_unlock(_mutex);                                            \
+        uvm_record_unlock_out_of_order(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
 // Unlock w/o any tracking. This should be extremely rare and *_no_tracking
@@ -771,25 +847,25 @@ static void __uvm_down(uvm_semaphore_t *semaphore)
 {
     down(&semaphore->sem);
 }
-#define uvm_down(sem) ({                                \
-        typeof(sem) _sem = (sem);                       \
-        uvm_record_lock(_sem, UVM_LOCK_MODE_SHARED);    \
-        __uvm_down(_sem);                               \
+#define uvm_down(sem) ({                                   \
+        typeof(sem) _sem = (sem);                          \
+        uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
+        __uvm_down(_sem);                                  \
     })
 
 static void __uvm_up(uvm_semaphore_t *semaphore)
 {
     up(&semaphore->sem);
 }
-#define uvm_up(sem) ({                                  \
-        typeof(sem) _sem = (sem);                       \
-        __uvm_up(_sem);                                 \
-        uvm_record_unlock(_sem, UVM_LOCK_MODE_SHARED);  \
+#define uvm_up(sem) ({                                       \
+        typeof(sem) _sem = (sem);                            \
+        __uvm_up(_sem);                                      \
+        uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
     })
-#define uvm_up_out_of_order(sem) ({                                      \
-        typeof(sem) _sem = (sem);                                        \
-        __uvm_up(_sem);                                                  \
-        uvm_record_unlock_out_of_order(_sem, UVM_LOCK_MODE_SHARED);      \
+#define uvm_up_out_of_order(sem) ({                                       \
+        typeof(sem) _sem = (sem);                                         \
+        __uvm_up(_sem);                                                   \
+        uvm_record_unlock_out_of_order(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
     })
 
 // A regular spinlock
@@ -817,9 +893,9 @@ typedef struct
 
 // Asserts that the spinlock is held. Notably the macros below support both
 // types of spinlocks.
-#define uvm_assert_spinlock_locked(spinlock) ({                                                         \
-        typeof(spinlock) _lock = (spinlock);                                                            \
-        UVM_ASSERT(spin_is_locked(&_lock->lock) && uvm_check_locked(_lock, UVM_LOCK_MODE_EXCLUSIVE));   \
+#define uvm_assert_spinlock_locked(spinlock) ({                                                             \
+        typeof(spinlock) _lock = (spinlock);                                                                \
+        UVM_ASSERT(spin_is_locked(&_lock->lock) && uvm_check_locked(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
     })
 
 #define uvm_assert_spinlock_unlocked(spinlock) UVM_ASSERT(!spin_is_locked(&(spinlock)->lock))
@@ -839,10 +915,10 @@ static void __uvm_spin_lock(uvm_spinlock_t *spinlock)
     spin_lock(&spinlock->lock);
     uvm_assert_spinlock_locked(spinlock);
 }
-#define uvm_spin_lock(lock) ({                              \
-        typeof(lock) _lock = (lock);                        \
-        uvm_record_lock(_lock, UVM_LOCK_MODE_EXCLUSIVE);    \
-        __uvm_spin_lock(_lock);                             \
+#define uvm_spin_lock(lock) ({                                 \
+        typeof(lock) _lock = (lock);                           \
+        uvm_record_lock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
+        __uvm_spin_lock(_lock);                                \
     })
 
 static void __uvm_spin_unlock(uvm_spinlock_t *spinlock)
@@ -850,10 +926,10 @@ static void __uvm_spin_unlock(uvm_spinlock_t *spinlock)
     uvm_assert_spinlock_locked(spinlock);
     spin_unlock(&spinlock->lock);
 }
-#define uvm_spin_unlock(lock) ({                            \
-        typeof(lock) _lock = (lock);                        \
-        __uvm_spin_unlock(_lock);                           \
-        uvm_record_unlock(_lock, UVM_LOCK_MODE_EXCLUSIVE);  \
+#define uvm_spin_unlock(lock) ({                                 \
+        typeof(lock) _lock = (lock);                             \
+        __uvm_spin_unlock(_lock);                                \
+        uvm_record_unlock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
 static void uvm_spin_lock_irqsave_init(uvm_spinlock_irqsave_t *spinlock, uvm_lock_order_t lock_order)
@@ -874,10 +950,10 @@ static void __uvm_spin_lock_irqsave(uvm_spinlock_irqsave_t *spinlock)
     spinlock->irq_flags = irq_flags;
     uvm_assert_spinlock_locked(spinlock);
 }
-#define uvm_spin_lock_irqsave(lock) ({                      \
-        typeof(lock) _lock = (lock);                        \
-        uvm_record_lock(_lock, UVM_LOCK_MODE_EXCLUSIVE);    \
-        __uvm_spin_lock_irqsave(_lock);                     \
+#define uvm_spin_lock_irqsave(lock) ({                         \
+        typeof(lock) _lock = (lock);                           \
+        uvm_record_lock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
+        __uvm_spin_lock_irqsave(_lock);                        \
     })
 
 static void __uvm_spin_unlock_irqrestore(uvm_spinlock_irqsave_t *spinlock)
@@ -887,10 +963,10 @@ static void __uvm_spin_unlock_irqrestore(uvm_spinlock_irqsave_t *spinlock)
     uvm_assert_spinlock_locked(spinlock);
     spin_unlock_irqrestore(&(spinlock)->lock, irq_flags);
 }
-#define uvm_spin_unlock_irqrestore(lock) ({                 \
-        typeof(lock) _lock = (lock);                        \
-        __uvm_spin_unlock_irqrestore(_lock);                \
-        uvm_record_unlock(_lock, UVM_LOCK_MODE_EXCLUSIVE);  \
+#define uvm_spin_unlock_irqrestore(lock) ({                      \
+        typeof(lock) _lock = (lock);                             \
+        __uvm_spin_unlock_irqrestore(_lock);                     \
+        uvm_record_unlock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
 // Bit locks are 'compressed' mutexes which take only 1 bit per lock by virtue
@@ -912,11 +988,11 @@ void uvm_bit_locks_deinit(uvm_bit_locks_t *bit_locks);
 // TODO: Bug 1766601:
 //  - assert for the right ownership (defining the owner might be tricky in
 //    the kernel).
-#define uvm_assert_bit_locked(bit_locks, bit) ({                        \
-    typeof(bit_locks) _bit_locks = (bit_locks);                         \
-    typeof(bit) _bit = (bit);                                           \
-    UVM_ASSERT(test_bit(_bit, _bit_locks->bits));                       \
-    UVM_ASSERT(uvm_check_locked(_bit_locks, UVM_LOCK_MODE_EXCLUSIVE));  \
+#define uvm_assert_bit_locked(bit_locks, bit) ({                             \
+    typeof(bit_locks) _bit_locks = (bit_locks);                              \
+    typeof(bit) _bit = (bit);                                                \
+    UVM_ASSERT(test_bit(_bit, _bit_locks->bits));                            \
+    UVM_ASSERT(uvm_check_locked(_bit_locks, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
 })
 
 #define uvm_assert_bit_unlocked(bit_locks, bit) ({                      \
@@ -933,11 +1009,11 @@ static void __uvm_bit_lock(uvm_bit_locks_t *bit_locks, unsigned long bit)
     UVM_ASSERT_MSG(res == 0, "Uninterruptible task interrupted: %d\n", res);
     uvm_assert_bit_locked(bit_locks, bit);
 }
-#define uvm_bit_lock(bit_locks, bit) ({                     \
-    typeof(bit_locks) _bit_locks = (bit_locks);             \
-    typeof(bit) _bit = (bit);                               \
-    uvm_record_lock(_bit_locks, UVM_LOCK_MODE_EXCLUSIVE);   \
-    __uvm_bit_lock(_bit_locks, _bit);                       \
+#define uvm_bit_lock(bit_locks, bit) ({                         \
+    typeof(bit_locks) _bit_locks = (bit_locks);                 \
+    typeof(bit) _bit = (bit);                                   \
+    uvm_record_lock(_bit_locks, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
+    __uvm_bit_lock(_bit_locks, _bit);                           \
 })
 
 static void __uvm_bit_unlock(uvm_bit_locks_t *bit_locks, unsigned long bit)
@@ -951,11 +1027,11 @@ static void __uvm_bit_unlock(uvm_bit_locks_t *bit_locks, unsigned long bit)
     smp_mb__after_atomic();
     wake_up_bit(bit_locks->bits, bit);
 }
-#define uvm_bit_unlock(bit_locks, bit) ({                   \
-    typeof(bit_locks) _bit_locks = (bit_locks);             \
-    typeof(bit) _bit = (bit);                               \
-    __uvm_bit_unlock(_bit_locks, _bit);                     \
-    uvm_record_unlock(_bit_locks, UVM_LOCK_MODE_EXCLUSIVE); \
+#define uvm_bit_unlock(bit_locks, bit) ({                         \
+    typeof(bit_locks) _bit_locks = (bit_locks);                   \
+    typeof(bit) _bit = (bit);                                     \
+    __uvm_bit_unlock(_bit_locks, _bit);                           \
+    uvm_record_unlock(_bit_locks, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
 })
 
 #endif // __UVM8_LOCK_H__
