@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -1014,30 +1014,6 @@ out:
     return status;
 }
 
-uvm_mmu_page_table_alloc_t *uvm_page_tree_pdb(uvm_page_tree_t *tree)
-{
-    return &tree->root->phys_alloc;
-}
-
-uvm_chunk_sizes_mask_t uvm_mmu_kernel_chunk_sizes(uvm_gpu_t *gpu)
-{
-    return allocation_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_64K) |
-           allocation_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_128K);
-}
-
-uvm_chunk_sizes_mask_t uvm_mmu_user_chunk_sizes(uvm_gpu_t *gpu)
-{
-    uvm_chunk_sizes_mask_t sizes = page_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_64K)  |
-                                   page_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_128K) |
-                                   PAGE_SIZE;
-
-    // Although we may have to map PTEs smaller than PAGE_SIZE, user (managed)
-    // memory is never allocated with granularity smaller than PAGE_SIZE. Force
-    // PAGE_SIZE to be supported and the smallest allowed size so we don't have
-    // to handle allocating multiple chunks per page.
-    return sizes & PAGE_MASK;
-}
-
 static size_t range_vec_calc_range_count(uvm_page_table_range_vec_t *range_vec)
 {
     NvU64 pde_coverage = uvm_mmu_pde_coverage(range_vec->tree, range_vec->page_size);
@@ -1481,7 +1457,23 @@ uvm_gpu_address_t uvm_mmu_gpu_address_for_big_page_physical(uvm_gpu_address_t ph
     return physical;
 }
 
-void uvm_mmu_init_gpu_peer_addresses(uvm_gpu_t *gpu)
+static void mmu_init_gpu_chunk_sizes(uvm_gpu_t *gpu)
+{
+    uvm_chunk_sizes_mask_t sizes = page_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_64K)  |
+                                   page_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_128K) |
+                                   PAGE_SIZE;
+
+    // Although we may have to map PTEs smaller than PAGE_SIZE, user (managed)
+    // memory is never allocated with granularity smaller than PAGE_SIZE. Force
+    // PAGE_SIZE to be supported and the smallest allowed size so we don't have
+    // to handle allocating multiple chunks per page.
+    gpu->mmu_user_chunk_sizes = sizes & PAGE_MASK;
+
+    gpu->mmu_kernel_chunk_sizes = allocation_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_64K) |
+                                  allocation_sizes_for_big_page_size(gpu, UVM_PAGE_SIZE_128K);
+}
+
+static void mmu_init_gpu_peer_addresses(uvm_gpu_t *gpu)
 {
     if (gpu->peer_copy_mode == UVM_GPU_PEER_COPY_MODE_VIRTUAL) {
         NvU32 i;
@@ -1495,6 +1487,12 @@ void uvm_mmu_init_gpu_peer_addresses(uvm_gpu_t *gpu)
     }
 }
 
+void uvm_mmu_init_gpu(uvm_gpu_t *gpu)
+{
+    mmu_init_gpu_chunk_sizes(gpu);
+    mmu_init_gpu_peer_addresses(gpu);
+}
+
 NV_STATUS uvm8_test_invalidate_tlb(UVM_TEST_INVALIDATE_TLB_PARAMS *params, struct file *filp)
 {
     NV_STATUS status;
