@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -250,6 +250,8 @@ void uvm_perf_prefetch_prenotify_fault_migrations(uvm_va_block_t *va_block,
     const uvm_page_mask_t *thrashing_pages = NULL;
     uvm_va_range_t *va_range = va_block->va_range;
 
+    uvm_assert_rwsem_locked(&va_range->va_space->lock);
+
     if (!g_uvm_perf_prefetch_enable)
         return;
 
@@ -267,8 +269,8 @@ void uvm_perf_prefetch_prenotify_fault_migrations(uvm_va_block_t *va_block,
     if (new_residency == UVM_CPU_ID || va_block->gpus[uvm_gpu_index(new_residency)] != NULL)
         resident_mask = uvm_va_block_resident_mask_get(va_block, new_residency);
 
-    // If this is a first-touch fault and the faulting processor is the preferred
-    // location, populate the whole VA block
+    // If this is a first-touch fault and the destination processor is the
+    // preferred location, populate the whole VA block
     if (uvm_processor_mask_empty(&va_block->resident) && new_residency == va_range->preferred_location) {
         uvm_page_mask_region_fill(&prefetch_info->prefetch_pages, uvm_va_block_region_from_block(va_block));
         goto done;
@@ -280,7 +282,10 @@ void uvm_perf_prefetch_prenotify_fault_migrations(uvm_va_block_t *va_block,
         uvm_page_mask_copy(&prefetch_info->bitmap_tree.pages, faulted_pages);
 
     // Get the big page size for the new residency
-    if (new_residency != UVM_CPU_ID) {
+    // Assume 64K size if the new residency is the CPU or no GPU va space is
+    // registered in the current process for this GPU.
+    if (new_residency != UVM_CPU_ID &&
+        uvm_processor_mask_test(&va_range->va_space->registered_gpu_va_spaces, new_residency)) {
         uvm_gpu_t *gpu = uvm_gpu_get(new_residency);
         prefetch_info->big_page_size = uvm_va_block_gpu_big_page_size(va_block, gpu);
     }
