@@ -412,7 +412,7 @@ static int nv_procfs_read_thrashing_stats(struct seq_file *s, void *v)
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(thrashing_stats);
+UVM_DEFINE_SINGLE_PROCFS_FILE(thrashing_stats);
 
 #define THRASHING_STATS_FILE_NAME "thrashing_stats"
 
