@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2017 NVIDIA Corporation
+    Copyright (c) 2017-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -517,7 +517,7 @@ static NV_STATUS test_pmm_sysmem_reverse_map(uvm_va_space_t *va_space, NvU64 add
 
     // Find a GPU with support for access counters, since it is required to add
     // or remove entries to the reverse map.
-    for_each_gpu_in_mask(gpu, &va_space->registered_gpus) {
+    for_each_va_space_gpu(gpu, va_space) {
         if (gpu->access_counters_supported) {
             // Initialize the reverse map.
             status = uvm_pmm_sysmem_mappings_init(gpu, &g_reverse_map);
