@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -758,12 +758,13 @@ out:
 
 NV_STATUS uvm8_test_pmm_query(UVM_TEST_PMM_QUERY_PARAMS *params, struct file *filp)
 {
+    uvm_va_space_t *va_space = uvm_va_space_get(filp);
     NV_STATUS status = NV_OK;
     uvm_gpu_t *gpu;
 
-    status = uvm_gpu_retain_by_uuid(&params->gpu_uuid, &gpu);
-    if (status != NV_OK)
-        return status;
+    gpu = uvm_va_space_retain_gpu_by_uuid(va_space, &params->gpu_uuid);
+    if (!gpu)
+        return NV_ERR_INVALID_DEVICE;
 
     switch (params->key) {
         case UVM_TEST_CHUNK_SIZE_GET_USER_SIZE:
@@ -809,15 +810,16 @@ out:
 
 NV_STATUS uvm8_test_pmm_check_leak(UVM_TEST_PMM_CHECK_LEAK_PARAMS *params, struct file *filp)
 {
+    uvm_va_space_t *va_space = uvm_va_space_get(filp);
     NV_STATUS status = NV_OK;
     uvm_gpu_t *gpu;
 
     if (params->alloc_limit < -1)
         return NV_ERR_INVALID_ARGUMENT;
 
-    status = uvm_gpu_retain_by_uuid(&params->gpu_uuid, &gpu);
-    if (status != NV_OK)
-        return status;
+    gpu = uvm_va_space_retain_gpu_by_uuid(va_space, &params->gpu_uuid);
+    if (!gpu)
+        return NV_ERR_INVALID_DEVICE;
 
     status = check_leak(gpu, params->chunk_size, params->alloc_limit, &params->allocated);
 
@@ -1226,7 +1228,7 @@ static NV_STATUS test_chunk_with_elevated_page(uvm_gpu_t *gpu)
     parent_root = parent_chunk;
     parent_chunk = NULL;
 
-    // Free some of the chunks 
+    // Free some of the chunks
     for (i = 0; i < num_chunks/2; i++) {
         UVM_ASSERT(chunks[i]);
         uvm_pmm_gpu_free(pmm, chunks[i], NULL);
@@ -1234,7 +1236,7 @@ static NV_STATUS test_chunk_with_elevated_page(uvm_gpu_t *gpu)
     }
 
     // Now try alloc a chunk of that size.
-    // Expect that the allocation will fail or return a chunk with a 
+    // Expect that the allocation will fail or return a chunk with a
     // different parent.
     status = chunk_alloc_check(pmm,
                                1,
