@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -161,25 +161,39 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
     else
         user_channel->instance_ptr.aperture = UVM_APERTURE_VID;
 
-    user_channel->instance_ptr.address   = channel_info.base;
-    user_channel->rm_retained_channel    = rm_retained_channel;
-    user_channel->hw_channel_id          = channel_info.chId;
-    user_channel->num_resources          = channel_info.resourceCount;
-    user_channel->engine_type            = channel_info.channelEngineType;
-    user_channel->in_subctx              = channel_info.bInSubctx == NV_TRUE;
-    user_channel->subctx_id              = channel_info.subctxId;
-    user_channel->in_tsg                 = channel_info.bTsgChannel == NV_TRUE;
-    user_channel->tsg_id                 = channel_info.tsgId;
-    user_channel->work_submission_token  = channel_info.workSubmissionToken;
-    user_channel->work_submission_offset = channel_info.workSubmissionOffset;
-    user_channel->clear_faulted_token    = channel_info.clearFaultedToken;
+    user_channel->instance_ptr.address    = channel_info.base;
+    user_channel->rm_retained_channel     = rm_retained_channel;
+    user_channel->hw_channel_id           = channel_info.chId;
+    user_channel->num_resources           = channel_info.resourceCount;
+    user_channel->engine_type             = channel_info.channelEngineType;
+    user_channel->in_subctx               = channel_info.bInSubctx == NV_TRUE;
+    user_channel->subctx_id               = channel_info.subctxId;
+    user_channel->tsg.valid               = channel_info.bTsgChannel == NV_TRUE;
+    user_channel->tsg.id                  = channel_info.tsgId;
+    user_channel->tsg.max_subctx_count    = channel_info.tsgMaxSubctxCount;
+    user_channel->work_submission_token   = channel_info.workSubmissionToken;
+    user_channel->work_submission_offset  = channel_info.workSubmissionOffset;
+    user_channel->clear_faulted_token     = channel_info.clearFaultedToken;
+
+
+
+
+
+
+
 
 
 
 
     // CE does not support subcontexts
-    if (user_channel->in_subctx)
+    if (user_channel->in_subctx) {
         UVM_ASSERT(user_channel->engine_type != UVM_GPU_CHANNEL_ENGINE_TYPE_CE);
+        UVM_ASSERT(user_channel->tsg.valid);
+        UVM_ASSERT(user_channel->subctx_id < user_channel->tsg.max_subctx_count);
+    }
+
+    if (user_channel->tsg.valid)
+        UVM_ASSERT(user_channel->tsg.max_subctx_count <= gpu->max_subcontexts);
 
     // If num_resources == 0, as can happen with CE channels, we ignore base and
     // length.
@@ -232,7 +246,7 @@ static uvm_va_range_t *find_va_range(uvm_user_channel_t *new_user_channel, NvP64
     uvm_va_range_t *range;
 
     // We can only allow sharing within a TSG
-    if (!new_user_channel->in_tsg)
+    if (!new_user_channel->tsg.valid)
         return NULL;
 
     list_for_each_entry(range, &gpu_va_space->channel_va_ranges, channel.list_node) {
@@ -240,7 +254,7 @@ static uvm_va_range_t *find_va_range(uvm_user_channel_t *new_user_channel, NvP64
         UVM_ASSERT(range->channel.ref_count > 0);
 
         if (range->channel.tsg.valid &&
-            range->channel.tsg.id == new_user_channel->tsg_id &&
+            range->channel.tsg.id == new_user_channel->tsg.id &&
             range->channel.rm_descriptor == rm_descriptor)
             return range;
     }
@@ -336,8 +350,8 @@ static NV_STATUS create_va_range(uvm_user_channel_t *user_channel, NvU64 base, N
     range->channel.gpu_va_space     = gpu_va_space;
     range->channel.aperture         = aperture;
     range->channel.rm_descriptor    = resource->resourceDescriptor;
-    range->channel.tsg.valid        = user_channel->in_tsg;
-    range->channel.tsg.id           = user_channel->tsg_id;
+    range->channel.tsg.valid        = user_channel->tsg.valid;
+    range->channel.tsg.id           = user_channel->tsg.id;
     range->channel.is_global        = resource->globalResource;
     range->channel.ref_count        = 1;
     list_add(&range->channel.list_node, &gpu_va_space->channel_va_ranges);
@@ -367,8 +381,8 @@ static void destroy_va_ranges(uvm_user_channel_t *user_channel)
 
         UVM_ASSERT(resource_range->type == UVM_VA_RANGE_TYPE_CHANNEL);
         UVM_ASSERT(resource_range->channel.rm_descriptor == user_channel->resources[i].resourceDescriptor);
-        UVM_ASSERT(resource_range->channel.tsg.valid == user_channel->in_tsg);
-        UVM_ASSERT(resource_range->channel.tsg.id == user_channel->tsg_id);
+        UVM_ASSERT(resource_range->channel.tsg.valid == user_channel->tsg.valid);
+        UVM_ASSERT(resource_range->channel.tsg.id == user_channel->tsg.id);
 
         // Drop the ref count on each each range
         UVM_ASSERT(resource_range->channel.ref_count > 0);
@@ -429,8 +443,8 @@ static NV_STATUS bind_channel_resources(uvm_user_channel_t *user_channel)
         UVM_ASSERT(resource_range);
         UVM_ASSERT(resource_range->type == UVM_VA_RANGE_TYPE_CHANNEL);
         UVM_ASSERT(resource_range->channel.rm_descriptor == user_channel->resources[i].resourceDescriptor);
-        UVM_ASSERT(resource_range->channel.tsg.valid == user_channel->in_tsg);
-        UVM_ASSERT(resource_range->channel.tsg.id == user_channel->tsg_id);
+        UVM_ASSERT(resource_range->channel.tsg.valid == user_channel->tsg.valid);
+        UVM_ASSERT(resource_range->channel.tsg.id == user_channel->tsg.id);
 
         resource_va_list[i].resourceDescriptor = resource_range->channel.rm_descriptor;
         resource_va_list[i].resourceVa         = resource_range->node.start;
