@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2018 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -62,7 +62,7 @@ static NV_STATUS register_gpu_nvlink_peers(uvm_va_space_t *va_space, uvm_gpu_t *
 
     uvm_assert_rwsem_locked(&va_space->lock);
 
-    for_each_gpu_in_mask(other_gpu, &va_space->registered_gpus) {
+    for_each_va_space_gpu(other_gpu, va_space) {
         uvm_gpu_peer_t *peer_caps;
 
         if (other_gpu->id == gpu->id)
@@ -214,7 +214,7 @@ static void unregister_gpu(uvm_va_space_t *va_space,
     // Notably do this only after unregistering the GPU from VA ranges to make
     // sure there is no pending work using the peer mappings within the VA
     // blocks (in particular migrations using the peer identity mappings).
-    for_each_gpu_in_mask(peer_gpu, &va_space->registered_gpus) {
+    for_each_va_space_gpu(peer_gpu, va_space) {
         if (gpu == peer_gpu)
             continue;
 
@@ -318,9 +318,8 @@ static uvm_ats_ibm_mm_t *uvm_va_space_find_ats_mm(uvm_va_space_t *va_space)
     return NULL;
 }
 
-void uvm_va_space_destroy(struct file *filp)
+void uvm_va_space_destroy(uvm_va_space_t *va_space)
 {
-    uvm_va_space_t *va_space = uvm_va_space_get(filp);
     uvm_gpu_va_space_t *gpu_va_space;
     uvm_va_range_t *va_range, *va_range_next;
     uvm_gpu_t *gpu, *peer_gpu;
@@ -475,9 +474,6 @@ void uvm_va_space_destroy(struct file *filp)
 
     uvm_mutex_unlock(&g_uvm_global.global_lock);
 
-    filp->private_data = NULL;
-    filp->f_mapping = NULL;
-
     uvm_kvfree(va_space);
 }
 
@@ -527,6 +523,21 @@ uvm_gpu_t *uvm_va_space_get_gpu_by_uuid_with_gpu_va_space(uvm_va_space_t *va_spa
     return gpu;
 }
 
+uvm_gpu_t *uvm_va_space_retain_gpu_by_uuid(uvm_va_space_t *va_space, const NvProcessorUuid *gpu_uuid)
+{
+    uvm_gpu_t *gpu;
+
+    uvm_va_space_down_read(va_space);
+
+    gpu = uvm_va_space_get_gpu_by_uuid(va_space, gpu_uuid);
+    if (gpu)
+        uvm_gpu_retain(gpu);
+
+    uvm_va_space_up_read(va_space);
+
+    return gpu;
+}
+
 bool uvm_va_space_can_read_duplicate(uvm_va_space_t *va_space, uvm_gpu_t *changing_gpu)
 {
     uvm_processor_mask_t changing_gpu_mask;
@@ -583,7 +594,7 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
     }
 
     // Mixing Volta and Pascal GPUs is not supported on P9 systems.
-    for_each_gpu_in_mask(other_gpu, &va_space->registered_gpus) {
+    for_each_va_space_gpu(other_gpu, va_space) {
         if ((gpu->sysmem_link >= UVM_GPU_LINK_NVLINK_2 && other_gpu->sysmem_link <  UVM_GPU_LINK_NVLINK_2) ||
             (gpu->sysmem_link <  UVM_GPU_LINK_NVLINK_2 && other_gpu->sysmem_link >= UVM_GPU_LINK_NVLINK_2)) {
             status = NV_ERR_INVALID_DEVICE;
@@ -922,59 +933,31 @@ static NV_STATUS enable_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu_1, uvm_gp
     return status;
 }
 
-static NV_STATUS retain_pcie_peers_from_uuids(const NvProcessorUuid *gpu_uuid_1,
+// On success the GPUs and the P2P access have been retained, but the caller
+// must not assume that the GPUs are still registered in the VA space after the
+// call since the VA space lock is dropped.
+static NV_STATUS retain_pcie_peers_from_uuids(uvm_va_space_t *va_space,
+                                              const NvProcessorUuid *gpu_uuid_1,
                                               const NvProcessorUuid *gpu_uuid_2,
                                               uvm_gpu_t **gpu_1,
                                               uvm_gpu_t **gpu_2)
 {
-    // The UUIDs should have already been retained
-    *gpu_1 = uvm_gpu_get_by_uuid(gpu_uuid_1);
-    if (!*gpu_1)
-        return NV_ERR_INVALID_DEVICE;
+    NV_STATUS status = NV_OK;
 
-    *gpu_2 = uvm_gpu_get_by_uuid(gpu_uuid_2);
-    if (!*gpu_2)
-        return NV_ERR_INVALID_DEVICE;
+    uvm_va_space_down_read_rm(va_space);
 
-    if ((*gpu_1)->id == (*gpu_2)->id)
-        return NV_ERR_INVALID_DEVICE;
+    // The UUIDs should have already been registered
+    *gpu_1 = uvm_va_space_get_gpu_by_uuid(va_space, gpu_uuid_1);
+    *gpu_2 = uvm_va_space_get_gpu_by_uuid(va_space, gpu_uuid_2);
 
-    return uvm_gpu_retain_pcie_peer_access(*gpu_1, *gpu_2);
-}
+    if (*gpu_1 && *gpu_2 && ((*gpu_1)->id != (*gpu_2)->id))
+        status = uvm_gpu_retain_pcie_peer_access(*gpu_1, *gpu_2);
+    else
+        status = NV_ERR_INVALID_DEVICE;
 
-static NV_STATUS retain_nvlink_peers_from_uuids(const NvProcessorUuid *gpu_uuid_1,
-                                                const NvProcessorUuid *gpu_uuid_2,
-                                                uvm_gpu_t **gpu_1,
-                                                uvm_gpu_t **gpu_2)
-{
-    NV_STATUS status;
-    uvm_gpu_peer_t *peer_caps;
+    uvm_va_space_up_read_rm(va_space);
 
-    // The UUIDs should have already been retained
-    status = uvm_gpu_retain_by_uuid_locked(gpu_uuid_1, gpu_1);
-    if (status != NV_OK)
-        return status;
-
-    status = uvm_gpu_retain_by_uuid_locked(gpu_uuid_2, gpu_2);
-    if (status != NV_OK) {
-        uvm_gpu_release_locked(*gpu_1);
-        return status;
-    }
-
-    if ((*gpu_1)->id == (*gpu_2)->id) {
-        uvm_gpu_release_locked(*gpu_1);
-        uvm_gpu_release_locked(*gpu_2);
-        return NV_ERR_INVALID_DEVICE;
-    }
-
-    peer_caps = uvm_gpu_peer_caps(*gpu_1, *gpu_2);
-
-    if (peer_caps->link_type >= UVM_GPU_LINK_NVLINK_1)
-        return NV_OK;
-
-    uvm_gpu_release_locked(*gpu_1);
-    uvm_gpu_release_locked(*gpu_2);
-    return NV_ERR_INVALID_DEVICE;
+    return status;
 }
 
 static bool uvm_va_space_pcie_peer_enabled(uvm_va_space_t *va_space, uvm_gpu_t *gpu1, uvm_gpu_t *gpu2)
@@ -1221,13 +1204,9 @@ NV_STATUS uvm_va_space_register_gpu_va_space(uvm_va_space_t *va_space,
     uvm_ats_ibm_mm_t *ats_mm = NULL;
     LIST_HEAD(deferred_free_list);
 
-    status = uvm_gpu_retain_by_uuid(gpu_uuid, &gpu);
-    if (status != NV_OK) {
-        // Convert to the return code specified by uvm.h for bad UUIDs
-        if (status == NV_ERR_GPU_UUID_NOT_FOUND)
-            return NV_ERR_INVALID_DEVICE;
-        return status;
-    }
+    gpu = uvm_va_space_retain_gpu_by_uuid(va_space, gpu_uuid);
+    if (!gpu)
+        return NV_ERR_INVALID_DEVICE;
 
     status = create_gpu_va_space(gpu, user_rm_va_space, &gpu_va_space);
     if (status != NV_OK) {
@@ -1466,31 +1445,40 @@ uvm_processor_id_t uvm_processor_mask_find_closest_id(uvm_va_space_t *va_space,
     if (uvm_processor_mask_test(candidates, src))
         return src;
 
-    // NVLINK peers. Prefer direct peers over indirect peers
-    uvm_processor_mask_and(&mask, candidates, &va_space->has_nvlink[src]);
-    if (uvm_processor_mask_andnot(&mask, &mask, &va_space->indirect_peers[src]))
-        return uvm_processor_mask_find_first_id(&mask);
+    // NvLink peers
+    if (uvm_processor_mask_and(&mask, candidates, &va_space->has_nvlink[src])) {
+        uvm_processor_mask_t direct_peers;
 
-    if (uvm_processor_mask_and(&mask, candidates, &va_space->has_nvlink[src]))
-        return uvm_processor_mask_find_first_id(&mask);
+        // Direct peers, prioritizing GPU peers over CPU
+        if (uvm_processor_mask_andnot(&direct_peers, &mask, &va_space->indirect_peers[src])) {
+            id = uvm_processor_mask_find_first_gpu_id(&direct_peers);
+            return id == UVM_MAX_PROCESSORS? UVM_CPU_ID : id;
+        }
 
-    // PCIe peers
-    // TODO: Bug 1764943: Is copying from a PCI peer always better than copying from CPU?
-    if (src != UVM_CPU_ID) {
-        uvm_processor_mask_and(&mask, candidates, &va_space->can_access[src]);
-        id = uvm_processor_mask_find_next_id(&mask, UVM_CPU_ID + 1);
-        if (id != UVM_MAX_PROCESSORS)
-            return id;
+        // Indirect peers
+        UVM_ASSERT(src != UVM_CPU_ID);
+        UVM_ASSERT(!uvm_processor_mask_test(&mask, UVM_CPU_ID));
+
+        return uvm_processor_mask_find_first_gpu_id(&mask);
     }
 
-    // No GPUs with direct access are in the mask. Prioritize the CPU next since
-    // we always have direct access to it. Then if all else fails, just pick the
-    // next available processor.
-    //
-    // We can do both of these with a single find_first_id, since the CPU is
-    // always first. Note that if src is the CPU, we already checked whether the
-    // CPU was present above.
-    return uvm_processor_mask_find_first_id(candidates);
+    // If source is GPU, prioritize PCIe peers over CPU
+    if (uvm_processor_mask_and(&mask, candidates, &va_space->can_access[src])) {
+        // CPUs only have direct access to GPU memory over NVLINK, not PCIe, and
+        // should have been selected above
+        UVM_ASSERT(src != UVM_CPU_ID);
+
+        id = uvm_processor_mask_find_first_gpu_id(&mask);
+        return id == UVM_MAX_PROCESSORS? UVM_CPU_ID : id;
+    }
+
+    // If the CPU would be a candidate, it would have been chosen by now since
+    // it is directly accessible from all processors
+    UVM_ASSERT(!uvm_processor_mask_test(candidates, UVM_CPU_ID));
+
+    // No GPUs with direct access, or the CPU, are in the mask. Just pick the
+    // first GPU in the mask, if any.
+    return uvm_processor_mask_find_first_gpu_id(candidates);
 }
 
 static void uvm_deferred_free_object_channel(uvm_deferred_free_object_t *object, uvm_processor_mask_t *flushed_gpus)
@@ -1567,7 +1555,7 @@ NV_STATUS uvm_api_enable_peer_access(UVM_ENABLE_PEER_ACCESS_PARAMS *params, stru
     size_t table_index;
 
     uvm_mutex_lock(&g_uvm_global.global_lock);
-    status = retain_pcie_peers_from_uuids(&params->gpuUuidA, &params->gpuUuidB, &gpu_1, &gpu_2);
+    status = retain_pcie_peers_from_uuids(va_space, &params->gpuUuidA, &params->gpuUuidB, &gpu_1, &gpu_2);
     uvm_mutex_unlock(&g_uvm_global.global_lock);
     if (status != NV_OK)
         return status;
@@ -1658,17 +1646,21 @@ NV_STATUS uvm8_test_enable_nvlink_peer_access(UVM_TEST_ENABLE_NVLINK_PEER_ACCESS
     uvm_gpu_t *gpu_1 = NULL;
     uvm_gpu_t *gpu_2 = NULL;
     size_t table_index;
-
-    // Retain GPUs so that they don't go away during peer registration
-    uvm_mutex_lock(&g_uvm_global.global_lock);
-    status = retain_nvlink_peers_from_uuids(&params->gpuUuidA, &params->gpuUuidB, &gpu_1, &gpu_2);
-    uvm_mutex_unlock(&g_uvm_global.global_lock);
-
-    if (status != NV_OK)
-        return status;
+    uvm_gpu_peer_t *peer_caps = NULL;
 
     uvm_va_space_down_write(va_space);
 
+    gpu_1 = uvm_va_space_get_gpu_by_uuid(va_space, &params->gpuUuidA);
+    gpu_2 = uvm_va_space_get_gpu_by_uuid(va_space, &params->gpuUuidB);
+
+    if (gpu_1 && gpu_2 && (gpu_1->id != gpu_2->id))
+        peer_caps = uvm_gpu_peer_caps(gpu_1, gpu_2);
+
+    if (!peer_caps || peer_caps->link_type < UVM_GPU_LINK_NVLINK_1) {
+        uvm_va_space_up_write(va_space);
+        return NV_ERR_INVALID_DEVICE;
+    }
+
     table_index = uvm_gpu_peer_table_index(gpu_1->id, gpu_2->id);
 
     // NVLink peers are automatically enabled in the VA space at VA space
@@ -1682,9 +1674,6 @@ NV_STATUS uvm8_test_enable_nvlink_peer_access(UVM_TEST_ENABLE_NVLINK_PEER_ACCESS
 
     uvm_va_space_up_write(va_space);
 
-    uvm_gpu_release(gpu_1);
-    uvm_gpu_release(gpu_2);
-
     return status;
 }
 
