@@ -134,7 +134,7 @@
     do {                                \
         if (0)                          \
             printk(fmt, ##__VA_ARGS__); \
-    } while(0)
+    } while (0)
 
 // printk_ratelimited was added in 2.6.33 via commit
 // 8a64f336bc1d4aa203b138d29d5a9c414a9fbb47. If not available, we prefer not
@@ -179,26 +179,6 @@
 
 #define NV_UVM_GFP_FLAGS (GFP_KERNEL | __GFP_NORETRY)
 
-#if defined(NV_VM_INSERT_PAGE_PRESENT)
-#define NV_VM_INSERT_PAGE(vma, addr, page) \
-    vm_insert_page(vma, addr, page)
-#endif
-
-#if defined(NV_REMAP_PFN_RANGE_PRESENT)
-#define NV_REMAP_PAGE_RANGE(from, offset, x...) \
-    remap_pfn_range(vma, from, ((offset) >> PAGE_SHIFT), x)
-#elif defined(NV_REMAP_PAGE_RANGE_PRESENT)
-#if (NV_REMAP_PAGE_RANGE_ARGUMENT_COUNT == 5)
-#define NV_REMAP_PAGE_RANGE(x...) remap_page_range(vma, x)
-#elif (NV_REMAP_PAGE_RANGE_ARGUMENT_COUNT == 4)
-#define NV_REMAP_PAGE_RANGE(x...) remap_page_range(x)
-#else
-#error "NV_REMAP_PAGE_RANGE_ARGUMENT_COUNT value unrecognized!"
-#endif
-#else
-#error "NV_REMAP_PAGE_RANGE() undefined!"
-#endif
-
 #if !defined(NV_ADDRESS_SPACE_INIT_ONCE_PRESENT)
     void address_space_init_once(struct address_space *mapping);
 #endif
