@@ -27,9 +27,9 @@ static NV_STATUS nv_dma_map_contig(
     NvU64 *va
 )
 {
-    *va = pci_map_page(dma_map->dev, dma_map->pages[0], 0,
+    *va = pci_map_page(dma_map->pci_dev, dma_map->pages[0], 0,
             dma_map->page_count * PAGE_SIZE, PCI_DMA_BIDIRECTIONAL);
-    if (NV_PCI_DMA_MAPPING_ERROR(dma_map->dev, *va))
+    if (pci_dma_mapping_error(dma_map->pci_dev, *va))
     {
         return NV_ERR_OPERATING_SYSTEM;
     }
@@ -39,12 +39,9 @@ static NV_STATUS nv_dma_map_contig(
     if (!IS_DMA_ADDRESSABLE(nv, *va) ||
         !IS_DMA_ADDRESSABLE(nv, *va + (dma_map->page_count * PAGE_SIZE - 1)))
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: DMA address not in addressable range of device "
-                "%04x:%02x:%02x (0x%llx-0x%llx, 0x%llx-0x%llx)\n",
-                NV_PCI_DOMAIN_NUMBER(dma_map->dev),
-                NV_PCI_BUS_NUMBER(dma_map->dev),
-                NV_PCI_SLOT_NUMBER(dma_map->dev),
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "DMA address not in addressable range of device "
+                "(0x%llx-0x%llx, 0x%llx-0x%llx)\n",
                 *va, *va + (dma_map->page_count * PAGE_SIZE - 1),
                 nv->dma_addressable_start,
                 nv->dma_addressable_limit);
@@ -57,7 +54,7 @@ static NV_STATUS nv_dma_map_contig(
 
 static void nv_dma_unmap_contig(nv_dma_map_t *dma_map)
 {
-    pci_unmap_page(dma_map->dev, dma_map->mapping.contig.dma_addr,
+    pci_unmap_page(dma_map->pci_dev, dma_map->mapping.contig.dma_addr,
             dma_map->page_count * PAGE_SIZE, PCI_DMA_BIDIRECTIONAL);
 }
 
@@ -140,7 +137,7 @@ NV_STATUS nv_create_dma_map_scatterlist(nv_dma_map_t *dma_map)
     defined(NV_DOM0_KERNEL_PRESENT)
         {
             NvU64 page_idx = NV_DMA_SUBMAP_IDX_TO_PAGE_IDX(i);
-            nv_fill_scatterlist(NV_DMA_SUBMAP_SCATTERLIST(submap),
+            nv_fill_scatterlist(submap->sgt.sgl,
                 &dma_map->pages[page_idx], submap->page_count);
         }
 #endif
@@ -166,9 +163,9 @@ NV_STATUS nv_map_dma_map_scatterlist(nv_dma_map_t *dma_map)
 
     NV_FOR_EACH_DMA_SUBMAP(dma_map, submap, i)
     {
-        submap->sg_map_count = pci_map_sg(dma_map->dev,
-                NV_DMA_SUBMAP_SCATTERLIST(submap),
-                NV_DMA_SUBMAP_SCATTERLIST_LENGTH(submap),
+        submap->sg_map_count = pci_map_sg(dma_map->pci_dev,
+                submap->sgt.sgl,
+                submap->sgt.orig_nents,
                 PCI_DMA_BIDIRECTIONAL);
         if (submap->sg_map_count == 0)
         {
@@ -197,8 +194,8 @@ void nv_unmap_dma_map_scatterlist(nv_dma_map_t *dma_map)
             break;
         }
 
-        pci_unmap_sg(dma_map->dev, NV_DMA_SUBMAP_SCATTERLIST(submap),
-                NV_DMA_SUBMAP_SCATTERLIST_LENGTH(submap),
+        pci_unmap_sg(dma_map->pci_dev, submap->sgt.sgl,
+                submap->sgt.orig_nents,
                 PCI_DMA_BIDIRECTIONAL);
     }
 }
@@ -215,7 +212,7 @@ void nv_destroy_dma_map_scatterlist(nv_dma_map_t *dma_map)
             break;
         }
 
-        NV_FREE_DMA_SUBMAP_SCATTERLIST(submap);
+        sg_free_table(&submap->sgt);
     }
 
     os_free_mem(dma_map->mapping.discontig.submaps);
@@ -233,7 +230,7 @@ void nv_load_dma_map_scatterlist(
 
     NV_FOR_EACH_DMA_SUBMAP(dma_map, submap, i)
     {
-        NV_FOR_EACH_DMA_SUBMAP_SG(submap, sg, j)
+        for_each_sg(submap->sgt.sgl, sg, submap->sg_map_count, j)
         {
             /*
              * It is possible for pci_map_sg() to merge scatterlist entries, so
@@ -262,16 +259,16 @@ static NV_STATUS nv_dma_map_scatterlist(
     status = nv_create_dma_map_scatterlist(dma_map);
     if (status != NV_OK)
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: Failed to allocate DMA mapping scatterlist!\n");
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Failed to allocate DMA mapping scatterlist!\n");
         return status;
     }
 
     status = nv_map_dma_map_scatterlist(dma_map);
     if (status != NV_OK)
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: failed to create a DMA mapping!\n");
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Failed to create a DMA mapping!\n");
         nv_destroy_dma_map_scatterlist(dma_map);
         return status;
     }
@@ -282,12 +279,9 @@ static NV_STATUS nv_dma_map_scatterlist(
     {
         if (!IS_DMA_ADDRESSABLE(nv, va_array[i]))
         {
-            nv_printf(NV_DBG_ERRORS,
-                    "NVRM: DMA address not in addressable range of device "
-                    "%04x:%02x:%02x (0x%llx, 0x%llx-0x%llx)\n",
-                    NV_PCI_DOMAIN_NUMBER(dma_map->dev),
-                    NV_PCI_BUS_NUMBER(dma_map->dev),
-                    NV_PCI_SLOT_NUMBER(dma_map->dev),
+            NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                    "DMA address not in addressable range of device "
+                    "(0x%llx, 0x%llx-0x%llx)\n",
                     va_array[i], nv->dma_addressable_start,
                     nv->dma_addressable_limit);
             nv_dma_unmap_scatterlist(dma_map);
@@ -400,20 +394,20 @@ NV_STATUS NV_API_CALL nv_dma_map_pages(
 
     if (page_count > os_get_num_phys_pages())
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: DMA mapping request too large!\n");
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "DMA mapping request too large!\n");
         return NV_ERR_INVALID_REQUEST;
     }
 
     status = os_alloc_mem((void **)&dma_map, sizeof(nv_dma_map_t));
     if (status != NV_OK)
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: Failed to allocate nv_dma_map_t!\n");
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Failed to allocate nv_dma_map_t!\n");
         return status;
     }
 
-    dma_map->dev = nvl->dev;
+    dma_map->pci_dev = nvl->pci_dev;
     dma_map->pages = *priv;
     dma_map->page_count = page_count;
     dma_map->contiguous = contig;
@@ -470,15 +464,15 @@ NV_STATUS NV_API_CALL nv_dma_unmap_pages(
 
     if (page_count > os_get_num_phys_pages())
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: DMA unmapping request too large!\n");
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "DMA unmapping request too large!\n");
         return NV_ERR_INVALID_REQUEST;
     }
 
     if (page_count != dma_map->page_count)
     {
-        nv_printf(NV_DBG_WARNINGS,
-                "NVRM: Requested to DMA unmap %llu pages, but there are %llu "
+        NV_DEV_PRINTF(NV_DBG_WARNINGS, nv,
+                "Requested to DMA unmap %llu pages, but there are %llu "
                 "in the mapping\n", page_count, dma_map->page_count);
         return NV_ERR_INVALID_REQUEST;
     }
@@ -525,8 +519,8 @@ NV_STATUS NV_API_CALL nv_dma_map_alloc
     status = os_alloc_mem((void **)&pages, pages_size);
     if (status != NV_OK)
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: Failed to allocate page array for DMA mapping!\n");
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Failed to allocate page array for DMA mapping!\n");
         return status;
     }
 
@@ -604,7 +598,7 @@ static NvBool nv_dma_is_map_resource_implemented
 )
 {
 #if defined(NV_DMA_MAP_RESOURCE_PRESENT)
-    const struct dma_map_ops *ops = get_dma_ops(&nvl->dev->dev);
+    const struct dma_map_ops *ops = get_dma_ops(nvl->dev);
 
     if (ops == NULL)
     {
@@ -641,19 +635,19 @@ NV_STATUS NV_API_CALL nv_dma_map_peer
     NV_STATUS status;
 
     BUG_ON(bar_index >= NV_GPU_NUM_BARS);
-    res = &nvlpeer->dev->resource[bar_index];
+    res = &nvlpeer->pci_dev->resource[bar_index];
     if (res->start == 0)
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: resource %u not valid for GPU " NV_PCI_DEV_FMT,
-                bar_index, NV_PCI_DEV_FMT_ARGS(peer));
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Resource %u not valid",
+                bar_index);
         return NV_ERR_INVALID_REQUEST;
     }
 
     if ((*va < res->start) || ((*va + (page_count * PAGE_SIZE)) > res->end))
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: mapping requested (start = 0x%llx, page_count = 0x%llx)"
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Mapping requested (start = 0x%llx, page_count = 0x%llx)"
                 " outside of resource bounds (start = 0x%llx, end = 0x%llx)\n",
                 *va, page_count, res->start, res->end);
         return NV_ERR_INVALID_REQUEST;
@@ -708,12 +702,12 @@ NV_STATUS NV_API_CALL nv_dma_map_mmio
 
     mmio_addr = *va;
 
-    *va = dma_map_resource(&nvl->dev->dev, mmio_addr, page_count * PAGE_SIZE,
+    *va = dma_map_resource(nvl->dev, mmio_addr, page_count * PAGE_SIZE,
                            DMA_BIDIRECTIONAL, 0);
-    if (dma_mapping_error(&nvl->dev->dev, *va))
+    if (dma_mapping_error(nvl->dev, *va))
     {
-        nv_printf(NV_DBG_ERRORS,
-                "NVRM: failed to DMA map MMIO range [0x%llx-0x%llx]\n",
+        NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
+                "Failed to DMA map MMIO range [0x%llx-0x%llx]\n",
                 mmio_addr, mmio_addr + page_count * PAGE_SIZE - 1);
         return NV_ERR_OPERATING_SYSTEM;
     }
@@ -753,7 +747,7 @@ void NV_API_CALL nv_dma_unmap_mmio
 
     nv_dma_nvlink_addr_decompress(nv, &va, page_count, NV_TRUE);
 
-    dma_unmap_resource(&nvl->dev->dev, va, page_count * PAGE_SIZE,
+    dma_unmap_resource(nvl->dev, va, page_count * PAGE_SIZE,
                        DMA_BIDIRECTIONAL, 0);
 #endif
 }
