@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2011 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2011-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -661,7 +661,7 @@ int nvidia_p2p_dma_map_pages(
 failed_insert:
     nv_p2p_free_dma_mapping(*dma_mapping);
     dma_addresses = NULL;
-    dma_mapping = NULL;
+    *dma_mapping = NULL;
 
 failed:
     if (peer_nvl != NULL)
@@ -674,9 +674,10 @@ failed:
         os_free_mem(dma_addresses);
     }
 
-    if (dma_mapping != NULL)
+    if (*dma_mapping != NULL)
     {
-        os_free_mem(dma_mapping);
+        os_free_mem(*dma_mapping);
+        *dma_mapping = NULL;
     }
 
     nv_kmem_cache_free_stack(sp);
