@@ -856,17 +856,17 @@ nv_procfs_close_unbind_lock(
 
             if (nv->flags & NV_FLAG_UNBIND_LOCK)
             {
-                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock acquired");
+                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock acquired\n");
             }
             else
             {
-                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "Could not acquire UnbindLock");
+                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "Could not acquire UnbindLock\n");
             }
         }
         else if ((value == 0) && (nv->flags & NV_FLAG_UNBIND_LOCK))
         {
             nv->flags &= ~NV_FLAG_UNBIND_LOCK;
-            NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock released");
+            NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock released\n");
         }
         up(&nvl->ldata_lock);
     }
