@@ -15,47 +15,27 @@
 
 static inline void nv_set_contig_memory_uc(nvidia_pte_t *page_ptr, NvU32 num_pages)
 {
-    if (nv_update_memory_types)
-    {
 #if defined(NV_SET_MEMORY_UC_PRESENT)
-        struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-        unsigned long addr = (unsigned long)page_address(page);
-        set_memory_uc(addr, num_pages);
+    struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
+    unsigned long addr = (unsigned long)page_address(page);
+    set_memory_uc(addr, num_pages);
 #elif defined(NV_SET_PAGES_UC_PRESENT)
-        struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-        set_pages_uc(page, num_pages);
-#elif defined(NV_CHANGE_PAGE_ATTR_PRESENT)
-        struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-        pgprot_t prot = PAGE_KERNEL_NOCACHE;
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
-        pgprot_val(prot) &= __nv_supported_pte_mask;
+    struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
+    set_pages_uc(page, num_pages);
 #endif
-        change_page_attr(page, num_pages, prot);
-#endif
-    }
 }
 
 
 static inline void nv_set_contig_memory_wb(nvidia_pte_t *page_ptr, NvU32 num_pages)
 {
-    if (nv_update_memory_types)
-    {
 #if defined(NV_SET_MEMORY_UC_PRESENT)
-        struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-        unsigned long addr = (unsigned long)page_address(page);
-        set_memory_wb(addr, num_pages);
+    struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
+    unsigned long addr = (unsigned long)page_address(page);
+    set_memory_wb(addr, num_pages);
 #elif defined(NV_SET_PAGES_UC_PRESENT)
-        struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-        set_pages_wb(page, num_pages);
-#elif defined(NV_CHANGE_PAGE_ATTR_PRESENT)
-        struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-        pgprot_t prot = PAGE_KERNEL;
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
-        pgprot_val(prot) &= __nv_supported_pte_mask;
+    struct page *page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
+    set_pages_wb(page, num_pages);
 #endif
-        change_page_attr(page, num_pages, prot);
-#endif
-    }
 }
 
 static inline int nv_set_memory_array_type_present(NvU32 type)
@@ -103,21 +83,18 @@ static inline void nv_set_contig_memory_type(
     NvU32 type
 )
 {
-    if (nv_update_memory_types)
+    switch (type)
     {
-        switch (type)
-        {
-            case NV_MEMORY_UNCACHED:
-                nv_set_contig_memory_uc(page_ptr, num_pages);
-                break;
-            case NV_MEMORY_WRITEBACK:
-                nv_set_contig_memory_wb(page_ptr, num_pages);
-                break;
-            default:
-                nv_printf(NV_DBG_ERRORS,
-                    "NVRM: %s(): type %d unimplemented\n",
-                    __FUNCTION__, type);
-        }
+        case NV_MEMORY_UNCACHED:
+            nv_set_contig_memory_uc(page_ptr, num_pages);
+            break;
+        case NV_MEMORY_WRITEBACK:
+            nv_set_contig_memory_wb(page_ptr, num_pages);
+            break;
+        default:
+            nv_printf(NV_DBG_ERRORS,
+                "NVRM: %s(): type %d unimplemented\n",
+                __FUNCTION__, type);
     }
 }
 
@@ -125,38 +102,33 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
 {
     NvU32 i;
     NV_STATUS status;
-    unsigned long *pages;
+    unsigned long *pages = NULL;
     nvidia_pte_t *page_ptr;
     struct page *page;
 
-    if (nv_update_memory_types)
+    if (nv_set_memory_array_type_present(type))
     {
-        pages = NULL;
+        status = os_alloc_mem((void **)&pages,
+                    at->num_pages * sizeof(unsigned long));
+        if (status != NV_OK)
+            pages = NULL;
+    }
 
-        if (nv_set_memory_array_type_present(type))
+    if (pages)
+    {
+        for (i = 0; i < at->num_pages; i++)
         {
-            status = os_alloc_mem((void **)&pages,
-                        at->num_pages * sizeof(unsigned long));
-            if (status != NV_OK)
-                pages = NULL;
-        }
-
-        if (pages)
-        {
-            for (i = 0; i < at->num_pages; i++)
-            {
-                page_ptr = at->page_table[i];
-                page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
-                pages[i] = (unsigned long)page_address(page);
-            }
-            nv_set_memory_array_type(pages, at->num_pages, type);
-            os_free_mem(pages);
-        }
-        else
-        {
-            for (i = 0; i < at->num_pages; i++)
-                nv_set_contig_memory_type(at->page_table[i], 1, type);
+            page_ptr = at->page_table[i];
+            page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
+            pages[i] = (unsigned long)page_address(page);
         }
+        nv_set_memory_array_type(pages, at->num_pages, type);
+        os_free_mem(pages);
+    }
+    else
+    {
+        for (i = 0; i < at->num_pages; i++)
+            nv_set_contig_memory_type(at->page_table[i], 1, type);
     }
 }
 
@@ -222,7 +194,7 @@ static void nv_flush_cache(void *p)
 static inline void nv_flush_caches(void)
 {
 #if NV_REQUIRE_HEAVY_WEIGHT_FLUSH
-    NV_ON_EACH_CPU(nv_flush_cache, NULL);
+    on_each_cpu(nv_flush_cache, NULL, 1);
 #endif
 }
 
@@ -245,7 +217,7 @@ static unsigned int nv_compute_gfp_mask(
 )
 {
     unsigned int gfp_mask = NV_GFP_KERNEL;
-    struct pci_dev *dev = at->dev;
+    struct device *dev = at->dev;
 
     /*
      * If we know that SWIOTLB is enabled (and therefore we avoid calling the
@@ -255,10 +227,10 @@ static unsigned int nv_compute_gfp_mask(
      * to the first 4GB to avoid allocating pages outside of our device's
      * addressable limit.
      */
-    if (!nv_requires_dma_remap(nv) || nv_is_dma_direct(&dev->dev))
+    if (!nv || !nv_requires_dma_remap(nv) || nv_is_dma_direct(dev))
     {
         NvU64 max_sysmem_address = nv_get_max_sysmem_address();
-        if (dev && (dev->dma_mask < max_sysmem_address))
+        if (dev && dev->dma_mask && (*(dev->dma_mask) < max_sysmem_address))
         {
             gfp_mask = NV_GFP_DMA32;
         }
@@ -302,11 +274,11 @@ static NV_STATUS nv_alloc_coherent_pages(
     unsigned long virt_addr = 0;
     dma_addr_t bus_addr;
     nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
-    struct pci_dev *dev = nvl->dev;
+    struct device *dev = nvl->dev;
 
     gfp_mask = nv_compute_gfp_mask(nv, at);
 
-    virt_addr = (unsigned long)dma_alloc_coherent(&dev->dev,
+    virt_addr = (unsigned long)dma_alloc_coherent(dev,
                                                   at->num_pages * PAGE_SIZE,
                                                   &bus_addr,
                                                   gfp_mask | __GFP_COMP);
@@ -342,7 +314,7 @@ static void nv_free_coherent_pages(
 )
 {
     nvidia_pte_t *page_ptr;
-    struct pci_dev *dev = at->dev;
+    struct device *dev = at->dev;
 
     page_ptr = at->page_table[0];
 
@@ -353,7 +325,7 @@ static void nv_free_coherent_pages(
                                   NV_MEMORY_WRITEBACK);
     }
 
-    dma_free_coherent(&dev->dev, at->num_pages * PAGE_SIZE,
+    dma_free_coherent(dev, at->num_pages * PAGE_SIZE,
                       (void *)page_ptr->virt_addr, page_ptr->dma_addr);
 
     if (!NV_ALLOC_MAPPING_CACHED(at->flags))
@@ -371,7 +343,7 @@ NV_STATUS nv_alloc_contig_pages(
     unsigned int gfp_mask;
     unsigned long virt_addr = 0;
     NvU64 phys_addr;
-    struct pci_dev *dev = at->dev;
+    struct device *dev = at->dev;
 
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
@@ -504,7 +476,7 @@ NV_STATUS nv_alloc_system_pages(
     unsigned int gfp_mask;
     unsigned long virt_addr = 0;
     NvU64 phys_addr;
-    struct pci_dev *dev = at->dev;
+    struct device *dev = at->dev;
 
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %u: %u pages\n", __FUNCTION__, at->num_pages);
@@ -664,3 +636,30 @@ void nv_vm_unmap_pages(
 
     nv_vunmap(virt_addr, count);
 }
+
+void nv_address_space_init_once(struct address_space *mapping)
+{
+#if defined(NV_ADDRESS_SPACE_INIT_ONCE_PRESENT)
+    address_space_init_once(mapping);
+#else
+    memset(mapping, 0, sizeof(*mapping));
+    INIT_RADIX_TREE(&mapping->page_tree, GFP_ATOMIC);
+
+#if defined(NV_ADDRESS_SPACE_HAS_RWLOCK_TREE_LOCK)
+    //
+    // The .tree_lock member variable was changed from type rwlock_t, to
+    // spinlock_t, on 25 July 2008, by mainline commit
+    // 19fd6231279be3c3bdd02ed99f9b0eb195978064.
+    //
+    rwlock_init(&mapping->tree_lock);
+#else
+    spin_lock_init(&mapping->tree_lock);
+#endif
+
+    spin_lock_init(&mapping->i_mmap_lock);
+    INIT_LIST_HEAD(&mapping->private_list);
+    spin_lock_init(&mapping->private_lock);
+    INIT_RAW_PRIO_TREE_ROOT(&mapping->i_mmap);
+    INIT_LIST_HEAD(&mapping->i_mmap_nonlinear);
+#endif /* !NV_ADDRESS_SPACE_INIT_ONCE_PRESENT */
+}
