@@ -473,6 +473,11 @@ void NV_API_CALL os_get_current_tick(NvU64 *nseconds)
     *nseconds = ((NvU64)ts.tv_sec * NSEC_PER_SEC + (NvU64)ts.tv_nsec);
 }
 
+NvU64 NV_API_CALL os_get_tick_resolution(void)
+{
+    return (NvU64)jiffies_to_usecs(1) * NSEC_PER_USEC;
+}
+
 #else
 
 void NV_API_CALL os_get_current_tick(NvU64 *nseconds)
@@ -497,6 +502,15 @@ void NV_API_CALL os_get_current_tick(NvU64 *nseconds)
                  (NvU64)useconds * NSEC_PER_USEC);
 }
 
+NvU64 NV_API_CALL os_get_tick_resolution(void)
+{
+    /*
+     * os_get_current_tick() uses os_get_current_time(), which has
+     * microsecond resolution.
+     */
+    return 1000ULL;
+}
+
 #endif
 
 //---------------------------------------------------------------------------
